/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************** * * * * * * * * * * * * **************************/

#ifndef _QHACC_H
#define _QHACC_H

#include "qhacctable.h"

#include <qfont.h>
#include <qcolor.h>
#include <qobject.h>
#include <qstring.h>
#include <qdatetime.h>

typedef TableRow Ledger;
typedef TableRow Account;
typedef TableRow Transaction;

/****************************************************************/
/** QHACCDB                                                    **/
/**   Handle all database calls                                **/
/**                                                            **/
/****************************************************************/

class Connection;

class QHaccDB {
 public:
	QHaccDB();
	~QHaccDB();
	enum Table { TRANSACTIONS, ACCOUNTS, PREFERENCES, ACCOUNTTYPES,
							 MEMTRANS, LEDGERS };

	bool load( const QString& home );
	bool load( Table, QHaccTable& );
	bool save( const QString& home, QString& error );

	TableRow at( Table, uint idx ) const;
	int add( Table, const TableRow& );

	int updateWhere( Table, const TableSelect&, int, const TableCol& replace );
	int updateWhere( Table, int position, const TableCol& equals, 
									 const TableRow& replace );
	int updateWhere( Table, int checkpos, const TableCol& equals, 
									 int changepos, const TableCol& replace );

	bool getWhere( Table, TableSelect *, uint size, QHaccTable *& results );
	bool getWhere( Table, const TableSelect&, QHaccTable *& results );

	uint deleteWhere( Table, const TableSelect& );
	uint deleteWhere( Table, int position, const TableCol& equals );
	uint deleteWhere( Table, QHaccTable * );

	bool contains( Table, const TableCol&, uint& ) const;
	bool contains( Table, const TableRow&, uint& ) const;
	
	void indexOn( Table, int col, QHaccTableIndex *& );

	uint rows( Table ) const;
	TableCol maxID( Table ) const;
	TableCol minID( Table ) const;
	bool isEmpty( Table ) const;

	void startLoad( Table, uint =0 );
	void stopLoad( Table );
	bool autosave() const;

 private:
	static const int NUMTABLES;
	static const char * DBNS[];

	QHaccTable ** dbs; // an array of all the tables
	QHaccTable * preferences, * accounts, * transactions;
	QHaccTable * accounttypes, * memtrans, * ledgers;
	Connection * mysql;

	QHaccTable * table( Table ) const;
	QString iconv( Table, const TableRow& ) const;
};


/****************************************************************/
/** QHACC Engine                                               **/
/**   This is basically a combination of the AccountManager,   **/
/**   QHacc, and QHaccLoader classes from the 1.X days.        **/
/**                                                            **/
/**                                                            **/
/**                                                            **/
/**                                                            **/
/****************************************************************/

class QHacc : public QObject {
  Q_OBJECT
		public:
	// trans constants
  static const int TID, TNUM, TPAYEE, TMEMO, TSUM, TDATE,	TRECO,
		TACCTID, TSPLITGROUP, TLID, TCOLS;
	static const char * TCOLNAMES[];
	static const ColType TCOLTYPES[];
	
	// account constants
	static const int AINSTNAME, AINSTADDR1, AINSTADDR2, AINSTCITY, AINSTSTATE,
    AINSTZIP, AINSTPHONE, AINSTFAX, AINSTEMAIL, AINSTCONTACT, AINSTNOTES,
		ANAME, ANUM, AOBAL, ACBAL, ATRANSNUMS, ADEFAULTNUM, ATAXED, ACATEGORY,
		ATYPE, APID, ACOLS, AID, ABUDGET, ARBAL;
	static const char * ACOLNAMES[];
	static const ColType ACOLTYPES[];
	static const QString ASEP;

	// preferences constants
	static const int PPREF, PVALUE, PCOLS;
	static const char * PCOLNAMES[];
	static const ColType PCOLTYPES[];

	// accounttype constants
	static const int YID, YNAME, YLP, YCOLS;
	static const char * YCOLNAMES[];
	static const ColType YCOLTYPES[];
	//static const uint ASSET, LIABILITY, EQUITY, EXPENSE, REVENUE;
	//static const bool YVALS[];

	// ledger constants
  static const int LID, LNAME, LDESC, LCOLS;
	static const char * LCOLNAMES[];
	static const ColType LCOLTYPES[];

	// other values
	static const uint YREC, NREC, MREC;
	static const int TOTALS, MONTHLYB, MONTHLYD, JOURNAL, BUDGET;
	static const int DECIMALS, ROLLOVER;
	static const int LINE, TWOLINE,	BAR, TWOBAR, PIE, AVES;
	static const int AMERICAN, EUROPEAN, YEARFIRST, NEVER, NEW, ALL;

	static const QDate XDATE;

	QHacc();
  ~QHacc();
	
	QString getHome() const;
	bool homeIsLocal() const;
	bool setHome( const QString& QHACC_HOME );
	bool save( QString& error );

	// Account functions
	void addA( const Account& );
	void updateA( const Account&, const Account& );
	void removeA( const Account& );
	Account getA( const QString& );
	Account getA( uint );
	QString getFNameOfA( const Account& );
	void getAs( bool excludeCategories, QHaccTable *& );
	uint aDepth( uint );
	Account getBlankA() const;
	QString getTypeNameOfA( const Account& );
	int getABalOn( const Account&, const QDate& );
	int getARBalTo( const Account&, const QDate& );
	void makeAT( QHaccTable *& ret, int pkcol =-1 ) const;
	bool isLPA( const Account& );

	// Transaction functions
	uint addTSG( QHaccTable * );
	void removeTSG( const TableCol& sg );
	void updateTSG( const TableCol& sg, QHaccTable * newsplits );
	void setTRec( const Transaction&, uint );
	Transaction getT( uint );
	void makeTT( QHaccTable *& ret, int pkcol=-1 ) const;
	
	void getTWhere( TableSelect *, uint, QHaccTable *& );
	void getTForA( const Account&, QHaccTable *& );
	void getTSplits( const Transaction&, QHaccTable *& );
	void getTSplits( uint id, QHaccTable *& );
	Transaction getBlankTForAL( const Account&, const Ledger& ) const;

	void getMSubsetForA( const Account&, QHaccTable *& );
	void memorize( uint tid, bool add );
	void addMtoL( uint tid, QDate, const Ledger& );
	

	// Ledger functions
	void addL( const Ledger& );
	void updateL( const Ledger&, const Ledger& );
	void removeL( const Ledger& );
	void getLs( QHaccTable *& );
	Ledger getL( const QString& );
	Ledger getL( uint lid );
	Ledger getBlankL();

	// Preference functions
	QString getSP( const QString& ) const;
	QColor getCP( const QString& ) const;
	QDate getDP( const QString& ) const;
	QFont getWP( const QString& ) const;
	float getFP( const QString& ) const;
	bool getBP( const QString& ) const;
	int getIP( const QString& ) const;

	void setSP( const QString&, QString );
	void setCP( const QString&, QColor );
	void setDP( const QString&, QDate );
	void setWP( const QString&, QFont );
	void setFP( const QString&, float );
	void setBP( const QString&, bool );
	void setIP( const QString&, int );

	// AccountType functions
	void getATs( QHaccTable *& );

	// command-line functions
	void archive( const char *, bool, const char * );
	void report( const char *, int, const char * );
	void cron( const char *, const char * );
	void showAccounts( const char * );
	void showTrans( const char *, const char * );
	bool crondollars( QHaccTable * trans );

	bool writeTransToFile( uint tid, const QString& file, QString& err );

	static QString convMoney( int cents );
	static int convMoney( const QString& );
	 
 private:
	QString qhacchome;
	QHaccDB db;

	bool isetP( const QString&, QString );
	bool condenseSG( QHaccTable * ) const;
	Account calcBalOfA( const Account& );
	void getTForArchive( const Account &, QHaccTable *& );

 signals:
	void addedA( const Account& );
	void removedA( const Account& );
	void updatedA( const Account&, const Account& );
	void overBudget( const Account&, int bal );

	void addedT( const Transaction& );
	void removedT( const Transaction& );
	void updatedT( const Transaction& );

	void addedT();
	void removedT();
	void updatedT();

	void addedL( const Ledger& );
	void removedL( const Ledger& );
	void updatedL( const Ledger&, const Ledger& );

	void changedP( const QString&, QString );
	void changedP( const QString&, QColor );
	void changedP( const QString&, QDate );
	void changedP( const QString&, QFont );
	void changedP( const QString&, float );
	void changedP( const QString&, bool );
	void changedP( const QString&, int );

	void needSave( bool );
};

#endif
