/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************* * * * * * * * * * * * * ***************************/

#include "qhacc.h"
#include "qhacclineedits.h"
#include "qhaccsegmenter.h"
#include "qhaccgraphpainters.h"

#include <stdlib.h>

#include <qpainter.h>

const QString QHaccGraphPainter::EMPTYSTR=QT_TR_NOOP( "No Transactions" );

QHaccGraphPainter::QHaccGraphPainter( QHacc * e ){
	engine=e;
	engine->makeAT( accounts, QHacc::AID );

	baseline=zeroline=0;
	data=0;
  // get some random colors to draw into graphs
  // i picked 12 so a big graph will repeat colors every year
  // it's neccessary to keep track of these colors because
  // partial repaints will look very bad otherwise  
  colors=new QColor[12];
  for ( int i=0; i<12; i++ )
    colors[i]=QColor( rand()%100+150, rand()%100+150, rand()%100+150 );
}

QHaccGraphPainter::~QHaccGraphPainter(){
	delete accounts;
	delete [] colors;
	if( data ) delete data;
}

void QHaccGraphPainter::setData( uint lid, const QHaccTable& accts,
																 const QDate& s, const QDate& e ){
	ledgerid=lid;
	accounts->clear();
	accounts->load( accts );
	start=s;
	end=e;
	isetData();
}

void QHaccGraphPainter::mouseDates( const QPoint&, QDate& st,	QDate& en ) const {
	st=start;
	en=end;
}

void QHaccGraphPainter::describeMouse( const QPoint& p, bool& inBase,
																			 bool& overZero, Account& acct,
																			 QDate& st, QDate& en ) const{
	inBase=( p.y()>baseline );
	overZero=( p.y()<zeroline );
	mouseAccount( p, acct );
	mouseDates( p, st, en );
}

void QHaccGraphPainter::painted( const QPixmap& ){}
void QHaccGraphPainter::paint( QPainter * p, const QRect& size ){
	QPixmap pix( size.width(), size.height() );
	QPainter painter( &pix );
	painter.fillRect( pix.rect(), engine->getCP( "MAINCOLOR" ) );
	paintBase( &painter, size );
	paintMain( &painter, size );
	p->drawPixmap( size.left(), size.top(), pix );
	painted( pix );
}

void QHaccGraphPainter::getPainter( QHacc * e, int t, QHaccGraphPainter *& p ){
	if( t==QHacc::LINE || t==QHacc::BAR )
		p=new QHaccSingleValPainter( e, t );
	else if( t==QHacc::TWOLINE || t==QHacc::TWOBAR )
		p=new QHaccIvsEPainter( e, t );
	else p=new QHaccPiePainter( e );
}




const uint QHaccPiePainter::TITLE=    0;
const uint QHaccPiePainter::TOTAL=    1;
const uint QHaccPiePainter::MERGED=   2;
const uint QHaccPiePainter::MERGEROWS=12;


QHaccPiePainter::QHaccPiePainter( QHacc * e ) : QHaccGraphPainter( e ){
	ColType types[]={ CTSTRING, CTFLOAT, CTBOOL };
	const char * names[]={ "TITLE", "TOTAL", "MERGED" };
	data=new QHaccTable( types, names, 3, TITLE );
}
QHaccPiePainter::~QHaccPiePainter(){}

void QHaccPiePainter::isetData(){
	bool de=engine->getBP( "DOUBLEENTRY" );
	bool payees=engine->getBP( "GRAPHPIEPAYEES" );

	data->clear();
	float datatot=0;

	QHaccTable trans( QHacc::TCOLTYPES, QHacc::TCOLNAMES, QHacc::TCOLS );
	QHaccTableIndex * idx=0;

	for( uint i=0; i<accounts->rows(); i++ ){
		Account account=accounts->at( i );
		QHaccTable * atrans=0;
		TableSelect criteria[]={
			TableSelect( QHacc::TACCTID, account.get( QHacc::AID ) ),
			TableSelect( QHacc::TDATE, TableCol( start ), TableSelect::GE ),
			TableSelect( QHacc::TDATE, TableCol( end ), TableSelect::LT ),
			( ledgerid==0 ? TableSelect() : TableSelect( QHacc::TLID,
																									 TableCol( ledgerid ) ) )
		};
		engine->getTWhere( criteria, 4, atrans );

		if( de &&! payees ){
			for( uint j=0; j<atrans->rows(); j++ ){
				Transaction tr=atrans->at( j );
				
				QHaccTable * splits=0;
				engine->getTSplits( tr, splits );
				trans.load( *splits );
				delete splits;
			}
		}
		else trans.load( *atrans );
		
		delete atrans;
	} // accounts loop

	// if de, we want to graph based on split trans' TACCTID
	// else we want to graph based on trans' TPAYEE
	if( de && !payees ) trans.addIndexOn( QHacc::TACCTID, idx );
	else trans.addIndexOn( QHacc::TPAYEE, idx );
	
	// segment the transactions appropriately
	uint * pos=0, sz=0;
	QHaccSegmenter::segment( engine, idx, pos, sz );
	
	for( uint i=0; i<sz-1; i++ ){
		Transaction t=idx->at( pos[i] );
		TableCol name;
		
		// figure out the partition heading
		if( de && !payees ){
			Account account=engine->getA( t.getu( QHacc::TACCTID ) );
			name=TableCol( account.gets( QHacc::ANAME ) );
		}
		else name=TableCol( t.gets( QHacc::TPAYEE ) );
		
		// figure out the associated value
		float sum=0;
		for( uint j=pos[i]; j<pos[i+1]; j++ )
			sum+=idx->at( j ).gets( QHacc::TSUM ).toFloat();
		if( sum<0 ) sum=0-sum; // we only want positive values to graph
		
		datatot+=sum;
		TableCol total( sum );
		TableCol d[]={ name, total, TableCol( false ) };
		data->add( TableRow( d, 3 ) );
	}
	delete [] pos;
	
	// we really only care about percentages of the pie, so replace the sum
	// field with the appropriate percentage
	for( uint i=0; i<data->rows(); i++ ){
		TableRow row=data->at( i );
		float f=row.getf( TOTAL );
		TableCol nsum( f/datatot );
		data->updateWhere( TITLE, row.gets( TITLE ), TOTAL, nsum );
	}

	// PRUNING: we want to prune datapoints if their percentage
	// is below MERGEPCT or there are more rows than MERGEROWS
	data->addIndexOn( TOTAL, summer );
	
	TableCol total( ( float )0 );
	TableCol d[]={ TableCol( QT_TR_NOOP( "Other" ) ), total, TableCol( true ) };
	TableRow other( d, 3 );

	const float MERGEPCT=engine->getFP( "MERGEPCT" );
	// figure out if any datapoints are too small to graph
	if( !data->isEmpty() ){
		float perc=0;

		while( perc<MERGEPCT ){
			TableRow row=summer->at( 0 );
			perc=row.getf( 1 );
			if( perc<MERGEPCT ){
				other.set( 1, TableCol( perc+other.getf( 1 ) ) );
				data->deleteWhere( TITLE, row.gets( TITLE ) );
			}
		}
	}

	// if we still have too many account, so merge the smallest ones
	if( data->rows()>MERGEROWS-1 ){ // yep, we're pruning
		float sum=0;
		// the index sorts ascendingly, so remove rows from the front
		uint lim=data->rows()-MERGEROWS;
		for( uint i=0; i<=lim; i++ ){
			TableRow row=summer->at( 0 );
			sum+=row.getf( 1 );
			data->deleteWhere( TITLE, row.gets( TITLE ) );
		}
		other.set( 1, TableCol( sum ) );
	}
	
	if( other.getf( 1 )>0 ) data->add( TableRow( other ) );
}

void QHaccPiePainter::paintBase( QPainter * p, const QRect& size ){
	//	cout<<"piepainter base"<<endl;
	QString temp;
  QFontMetrics fm=p->fontMetrics();
	int fmh=fm.height();
	bool showTots=engine->getBP( "GRAPHSHOWTOTALS" );
	uint rows=data->rows();

	int baseh=5*fmh; // allow at most 5 rows for the legend
	zeroline=-1;
	baseline=size.height()-baseh;
	
	int wspot=size.left(), hspot=baseline;
	// width per account--each account get's 1/3 of the screen space for its name
	int wpa=size.width()/3;
	for( uint i=0; i<rows; i++ ){
		TableRow row=summer->at( ( rows-1 )-i );

		if( i%3==0 ){
			wspot=0;    // carriage return
			hspot+=fmh; // new line
		}
		else wspot+=wpa;

		QString perc=temp.setNum( row.getf( TOTAL )*100, 'f', 0 );
		int pw=fm.width( perc );
		if( pw<15 ) pw=15;
		int pw2=pw+2;
		
		// draw the colored square for the legend
		p->fillRect( wspot, hspot, pw, 15, colors[i] );
		// draw the appropriate name
		p->drawText( wspot+pw2, hspot, wpa-pw2, fmh, p->AlignLeft,
								 row.gets( TITLE ) );
		// draw the totals in the box, if necessary
		if( showTots ) p->drawText( wspot, hspot, pw, fmh, p->AlignCenter, perc );
	}
}

void QHaccPiePainter::paintMain( QPainter * p, const QRect& size ){
	// cout<<"piepainter main"<<endl;
	if( data->isEmpty() ){
		p->drawText( size.left(), size.top(), size.width(), size.height(), p->AlignCenter, EMPTYSTR );
		return;
	}

	int h=baseline;
	int w=size.width();

	int oldh=size.top();
	uint rows=data->rows();
	for ( uint i=0; i<rows; i++ ){
		p->setBrush( colors[i] );
		TableRow row=summer->at( ( rows-1 )-i );
		//summer->at( i, row );
		
		int hspot=( int )( row.getf( TOTAL )*5760 ); //360*16=5760
		p->drawPie( 10, 10, w-20, h-20, oldh, hspot );
		oldh+=hspot;
	}
}

void QHaccPiePainter::painted( const QPixmap& pix ){ 
	image=pix.convertToImage();
}

void QHaccPiePainter::mouseAccount( const QPoint&, Account& ) const {
	// the colors returned by image.pixel are slightly different from the 
	// colors in the colors array. Thus, we can't figure out accounts yet.

	/*
	// go through the colors, and see if we can find the right account
	bool de=engine->getBP( "DOUBLEENTRY" );

	QColor testcolor( image.pixel( p.x(), p.y() ) );
	cout<<"("<<testcolor.red()<<","<<testcolor.green()<<","<<testcolor.blue()<<")"<<endl;
	for( uint i=0; i<data->rows(); i++ ){
	cout<<"  against("<<colors[i].red()<<","<<colors[i].green()<<","<<colors[i].blue()<<")"<<endl;
		if( colors[i]==testcolor ){ // FOUND!
			cout<<"found at index "<<i<<endl;
			TableRow row=data->at( i );
			if( de ){
				if( !row.getb( MERGED ) ) acct=engine->getA( row.gets( TITLE ) );
			}
			else{
				TableSelect criteria[]={
					TableSelect( QHacc::TPAYEE, row.gets( TITLE ) ),
					TableSelect( QHacc::TDATE, TableCol( start ), TableSelect::GE ),
					TableSelect( QHacc::TDATE, TableCol( end ), TableSelect::LT ),
				};
			}
		}
	}
	*/
}


const uint QHaccMonthlyPainter::AID=      0;
const uint QHaccMonthlyPainter::NUMBER=   1;
const uint QHaccMonthlyPainter::INCREASES=2;
const uint QHaccMonthlyPainter::DECREASES=3;

QHaccMonthlyPainter::QHaccMonthlyPainter( QHacc * e, int t )
	: QHaccGraphPainter( e ){
	line=( t==QHacc::LINE || t==QHacc::TWOLINE );
	ivse=( t==QHacc::TWOLINE || t==QHacc::TWOBAR );

	ColType types[]={ CTUINT, CTUINT, CTFLOAT, CTFLOAT };
	const char * names[]={ "AID", "NUMBER", "INCREASES", "DECREASES" };
	data=new QHaccTable( types, names, 4 );
	monthstrs=new QString[0];
}

QHaccMonthlyPainter::~QHaccMonthlyPainter(){ delete [] monthstrs; }

void QHaccMonthlyPainter::isetData(){
	data->clear();
	delete [] monthstrs;
	hiVal=loVal=0;
	bool totals=( !( ivse || engine->getBP( "GRAPHDELTAS" ) ) );


	for( uint i=0; i<accounts->rows(); i++ ){
		Account a=accounts->at( i );
		TableSelect criteria[]={
			TableSelect( QHacc::TACCTID, a.get( QHacc::AID ) ),
			TableSelect( QHacc::TDATE, TableCol( start ), TableSelect::GE ),
			TableSelect( QHacc::TDATE, TableCol( end ), TableSelect::LT ),
			( ledgerid==0 ? TableSelect() : TableSelect( QHacc::TLID, 
																									 TableCol( ledgerid ) ) )
		};

		QHaccTable * trans=0;
		engine->getTWhere( criteria, 4, trans );

		/*
		cout<<"trans contains: "<<endl;
		for( uint i=0; i<trans->rows(); i++ ) cout<<"  "<<trans->at( i ).toString()<<endl;
		cout<<"end"<<endl;
		*/

		QHaccTableIndex * idx=0;
		trans->addIndexOn( QHacc::TDATE, idx );

		uint * pos=0, sz=0;		
		QHaccSegmenter::segment( engine, idx, start, end, pos, sz );

		
		//for( uint q=0; q<trans->rows(); q++ ) cout<<idx->at( q ).toString()<<endl;

		if( i==0 ){
			months=sz-1;
			monthstrs=new QString[months];
		}

		TableRow last;
		for( uint month=0; month<months; month++ ){
			if( i==0 ){ // get the headings on the first run
				QDate ndate=start.addMonths( month );
				monthstrs[month]=QHaccDateEdit::getShortDisplayString( ndate )+"- ";
				ndate=start.addMonths( month+1 );
				if( ndate>end ) ndate=end;
				monthstrs[month]+=
					QHaccDateEdit::getShortDisplayString( ndate.addDays( -1 ) );
			}

			//cout<<"  month is ="<<month<<"; pos[month]="<<pos[month]<<"-"<<pos[month+1]<<endl;
			// figure out the values
			float incrs=0, decrs=0;
			if( totals ){
				if( last.isNull() ) incrs=engine->getABalOn( a, start );
				else incrs=last.getf( INCREASES )+last.getf( DECREASES );
			}

			for( uint k=pos[month]; k<pos[month+1]; k++ ){
				Transaction t=idx->at( k );

				//cout<<"  k="<<k<<"  "<<t.toString()<<endl;
				
				float f=t.gets( QHacc::TSUM ).toFloat();
				if( f<0 ) decrs+=f;
				else incrs+=f;
			}

			if( ivse ){
				if( incrs>hiVal ) hiVal=incrs;
				if( decrs<loVal ) loVal=decrs;
			}
			else{
				float val=incrs+decrs;
				if( val>hiVal ) hiVal=val;
				if( val<loVal ) loVal=val;
			}
			TableCol d[]={ a.get( QHacc::AID ), TableCol( month ),
										 TableCol( incrs ), TableCol( decrs ) };
			last=TableRow( d, 4 );
      data->add( last );
		}
		delete trans;
		delete [] pos;
	}
	
	//for( uint i=0; i<months; i++ ) cout<<monthstrs[i]<<"\t"<<endl;
	QHaccTableIndex * junk=0;
	data->addIndexOn( AID, junk );
	data->addIndexOn( NUMBER, junk );

	/*
	for( uint i=0; i<data->rows(); i++ ){
		TableRow * row=0;
		data->at( i, row );
		cout<<row->toString()<<endl;
	}
	*/
}

void QHaccMonthlyPainter::mouseDates( const QPoint& p, QDate& st,
																			QDate& en ) const {
	uint lim=p.x()/wpm;
	st=start.addMonths( lim );
	en=start.addMonths( lim+1 ).addDays( -1 );
}


void QHaccMonthlyPainter::mouseAccount( const QPoint& p,
																				Account& acct ) const {
	uint n=p.x()/wpa;
	uint acctn=n%accounts->rows();
	acct=accounts->at( acctn );
}

void QHaccMonthlyPainter::paintBase( QPainter * p, const QRect& size ){
	if( months==0 ) return;

	wpm=size.width()/months;
	wpa=wpm/accounts->rows();
	//cout<<"monthlypainter base: wpam="<<wpm<<"; wpa="<<wpa<<endl;
	QFontMetrics fm=p->fontMetrics();
	const QColor GRAY=QColor( "darkgray" );
	int fmh=fm.height();
	int h=size.height(), w=size.width();


	int baseh=3*fmh;
	baseline=h-baseh;
	factor=( hiVal-loVal )/( baseline-2*fmh ); // fmh is wiggle room
	//cout<<"hiVal is "<<hiVal<<"; loVal is "<<loVal<<"; baseline is "<<baseline<<"; factor is "<<factor<<endl;
	if( factor==0 ) factor=1;
	zeroline=baseline-( int )( ( 0-loVal )/factor )-fmh; // more wiggling


  QPen pen=p->pen(); // save the original pen for later

	// draw the zero line in dark black
	p->setPen( QPen( "black" ) );	
	QString temp="0";
	p->drawText( 0, zeroline, temp );
	p->drawLine( 0, zeroline, w, zeroline );
	p->drawText( w-fm.width( temp ), zeroline, temp );

	// draw guidelines, if necessary
	if( engine->getBP( "GRAPHGUIDES" ) ){
		p->setPen( GRAY );
		const int NUMLINES=4;
		
		float spanpl=( hiVal-loVal )/( factor*NUMLINES );
		float liner=( zeroline-( hiVal/factor ) );
	
	  // the +1 puts a line at the lVal point
		for( int i=0; i<NUMLINES+1; i++ ){
		  temp.setNum( ( zeroline-liner )*factor, 'f', 0 );
			p->drawText( 0, liner, temp );
			p->drawLine( 0, liner, w, liner );
			p->drawText( w-fm.width( temp ), liner, temp );
			liner+=spanpl;
		}
	}

	// draw the month headers
	bool manyaccts=( accounts->rows()>1 );
	p->setPen( pen );
	for( uint i=0; i<months; i++ ){
		int wspot=wpm*i;
		QColor color=GRAY;
		
		if( manyaccts ){
			// alternate light, dark grays
		  if( i%2 ) color=color.light( 110 );
		}
		else color=colors[i%12];

		p->fillRect( wspot, baseline, wpm*( i+1 ), baseline+baseh, color );
		

		//if the column label is too long, just put a placeholder
		if ( fm.width( monthstrs[i] )>=( int )( wpm*2 ) ) temp="|";
		else temp=monthstrs[i];
			
		//column labels
		//			p->drawText( wspot-half, hspot, wpp, hespot,
		//AlignTop|AlignHCenter|WordBreak, temp );
		p->drawText( wspot, baseline, wpm, baseh,
		             p->AlignTop|p->AlignHCenter|p->WordBreak, temp );
	}
}

void QHaccMonthlyPainter::paintMain( QPainter * p, const QRect& size ){
	if( months==0 ) {
		p->drawText( size.left(), size.top(), size.width(), size.height(),
								 p->AlignCenter, EMPTYSTR );
		return;
	}

	bool drawSums=engine->getBP( "GRAPHSHOWTOTALS" );
	bool drawHeads=engine->getBP( "GRAPHHEADS" );
	bool manyaccts=( accounts->rows()>1 );
	uint arows=accounts->rows();

	// go through each account, and draw all its data before moving on
	// to the next account
	for( uint acctn=0; acctn<arows; acctn++ ){
		Account acct=accounts->at( acctn );

		QHaccTable * vals=0;
		data->getWhere( TableSelect( AID, acct.get( QHacc::AID ) ), vals );
		QHaccTableIndex * idx=0;
		vals->addIndexOn( NUMBER, idx );

		TableRow last;
		for( uint month=0; month<months; month++ ){
			QString title;
			if( month==0 && drawHeads && manyaccts )
				title=acct.gets( QHacc::ANAME );

			int wstart=wpm*month;
			int wspot=wstart+( acctn*wpa );
			

			TableRow row=idx->at( month );
			paintRow( p, last, row, wstart, wspot,
								colors[( manyaccts ? acctn : month )%12], drawSums, title );
			last=row;
		}

		delete vals;
	}
}

void QHaccMonthlyPainter::drawTitle( QPainter * p, const QString& title,
																		 int wstart, int wspot, int hspot ) const {
	QFontMetrics fm=p->fontMetrics();
	int fmh=fm.height();

	if( line ) p->drawText( wspot+5, hspot-fmh/2, title );
	else{
		p->setPen( QPen( "black" ) );
		hspot=zeroline-( int )( hiVal/factor );
		for( uint j=0; j<title.length(); j++ ){
			hspot+=fmh;
			p->drawText( wstart, hspot, wpa, fmh,
									 p->AlignCenter, title.mid( j, 1 ) );
		}
	}
}

void QHaccMonthlyPainter::drawSum( QPainter * p, float sum,
																	 int x, int y, int w, int h ) const {
	p->setPen( QPen( "black" ) );
	static QString temp;
	temp.setNum( sum, 'f', 0 );
	if( sum>0 ) y-=h; // put positive sums a little higher
	if( sum!=0 ) p->drawText( x, y, w, h, p->AlignCenter|p->DontClip, temp );
}






QHaccSingleValPainter::QHaccSingleValPainter( QHacc * e, int t )
	: QHaccMonthlyPainter( e, t ){}
QHaccSingleValPainter::~QHaccSingleValPainter(){}
void QHaccSingleValPainter::paintRow( QPainter * p, const TableRow& last, 
																			const TableRow& row, int wstart,
																			int wspot, const QColor& color,
																			bool drawTots, 
																			const QString& head ) const {
	//cout<<"PAINTROW\t"<<row->toString()<<"\t";
	//if( last ) cout<<last->toString();
	//else cout<<"no last row";
	//cout<<endl;
	
	float sum=row.getf( INCREASES )+row.getf( DECREASES );
	int hpos=zeroline-( int )( sum/factor );
	bool manyaccts=( accounts->rows()>1 );
	int sstart=wstart, send=wpm;
	int tstart=wstart;
	//p->setPen( QPen( color ) );
	//cout<<"  point is ("<<wpos<<","<<hpos<<")..."<<sum<<"*"<<factor<<"="<<sum*factor<<endl;

	if( line ){
		int wpos=wstart+( wpm/2 );
		tstart=wpos;
		if( manyaccts )	sstart=wstart;
		
		if( !last.isNull() ){
			if( manyaccts ) p->setPen( QPen( color, 3 ) );
			else p->setPen( QPen( "black", 3 ) );
			
			// draw a line from the last value to this one
			float lsum=last.getf( INCREASES )+last.getf( DECREASES );
			int lwpos=wpos-wpm;
			int lhpos=zeroline-( int )( lsum/factor );
			//cout<<"  last was ("<<lwpos<<","<<lhpos<<")..."<<lsum<<"*"<<factor<<"="<<sum*factor<<endl;
			p->drawLine( lwpos, lhpos, wpos, hpos );
			
			p->fillRect( lwpos-3, lhpos-3, 6, 6, QBrush( "black" ) );
		}
		
		// draw a little box over the datapoint
		p->fillRect( wpos-3, hpos-3, 6, 6, QBrush( "black" ) );
	}
	else{
		sstart=wspot;
		send=wpa;
		QRect rect( wspot, hpos, wpa, zeroline-hpos );
		QColor col;
		if( col==INCREASES ) col=color.light( 110 );
		else col=color.dark( 115 );
		
		p->fillRect( rect, QBrush( col ) );
		p->setPen( QPen( "black" ) );
		p->drawRect( rect );
	}
	
	if( !head.isNull() ) drawTitle( p, head, sstart, tstart, hpos );	

	if( drawTots )
		drawSum( p, sum, sstart, hpos, send, p->fontMetrics().height() );
}





QHaccIvsEPainter::QHaccIvsEPainter( QHacc * e, int t )
	: QHaccMonthlyPainter( e, t ){}
QHaccIvsEPainter::~QHaccIvsEPainter(){}
void QHaccIvsEPainter::paintRow( QPainter * p, const TableRow& last, 
																 const TableRow& row, int wstart, int wspot,
																 const QColor& color, bool drawTots,
																 const QString& head ) const {
	paintRow( p, last, row, wstart, wspot, color, drawTots, head, INCREASES );
	paintRow( p, last, row, wstart, wspot, color, drawTots, head, DECREASES );
}

void QHaccIvsEPainter::paintRow( QPainter * p, const TableRow& last, 
																 const TableRow& row, int wstart, int wspot,
																 const QColor& color, bool drawTots,
																 const QString& head, uint col ) const {
	//cout<<"PAINTROW\t"<<row->toString()<<"\t";
	//if( last ) cout<<last->toString();
	//else cout<<"no last row";
	//cout<<endl;

	float sum=row.getf( col );
	int hpos=zeroline-( int )( sum/factor );
	bool manyaccts=( accounts->rows()>1 );
	int sstart=wstart, send=wpm;
	int tstart=wstart;
	//p->setPen( QPen( color ) );
	//cout<<"  point is ("<<wpos<<","<<hpos<<")..."<<sum<<"*"<<factor<<"="<<sum*factor<<endl;

	if( line ){
		int wpos=wstart+( wpm/2 );
		tstart=wpos;
		if( manyaccts ) sstart=wstart;

		if( !last.isNull() ){
			if( manyaccts ) p->setPen( QPen( color, 3 ) );
			else p->setPen( QPen( "black", 3 ) );

			// draw a line from the last value to this one
			float lsum=last.getf( col );
			int lwpos=wpos-wpm;
			int lhpos=zeroline-( int )( lsum/factor );
			//cout<<"  last was ("<<lwpos<<","<<lhpos<<")..."<<lsum<<"*"<<factor<<"="<<sum*factor<<endl;
			p->drawLine( lwpos, lhpos, wpos, hpos );

			p->fillRect( lwpos-3, lhpos-3, 6, 6, QBrush( "black" ) );
		}

		// draw a little box over the datapoint
		p->fillRect( wpos-3, hpos-3, 6, 6, QBrush( "black" ) );

	}
	else{
		sstart=wspot;
		send=wpa;
		QRect rect( wspot, hpos, wpa, zeroline-hpos );
		QColor col;
		if( col==INCREASES ) col=color.light( 110 );
		else col=color.dark( 115 );

		p->fillRect( rect, QBrush( col ) );
		p->setPen( QPen( "black" ) );
		p->drawRect( rect );
	}
	
	if( !head.isNull() ) drawTitle( p, head, sstart, tstart, hpos );	

	if( drawTots )
		drawSum( p, sum, sstart, hpos, send, p->fontMetrics().height() );
}

