/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************* * * * * * * * * * * * * ***************************/

#include "qhaccview.h"
#include "qhaccutils.h"
#include "qhaccrecwin.h"
#include "qhacclineedits.h"
#include "qhaccrecwin.moc"

#include <qfont.h>
#include <qsize.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qpopupmenu.h>
#include <qpushbutton.h>

QHaccRecWin::QHaccRecWin( QHacc * e, const Ledger& l,  QWidget * p,
													const char * n )
	: QWidget( p, n, WDestructiveClose ) {
  
	engine=e;
	ledger=l;
	setFont( e->getWP( "FONT" ) );
  tView=new QHaccRecView( engine, this );

  enddate=new QHaccDateEdit( QDate(), this );
  enddate->setAlignment( AlignCenter );
  
	const QString zero=QString::number( 0.00 );
  ebal=new QHaccLineEdit( zero, this );
  ebal->setAlignment( AlignCenter );

  lblOBal=new QLabel( zero, this );
  lblDiff=new QLabel( zero, this );
  
  QPushButton * ok=new QPushButton( tr( "Finished" ), this );
  QPushButton * cancel=new QPushButton( tr( "Cancel" ), this );
  connect ( ok, SIGNAL( clicked() ), SLOT( accept() ) );
  connect ( cancel, SIGNAL( clicked() ), SLOT( cancel() ) );
  
  connect( enddate, SIGNAL( dateChanged( QDate ) ), SLOT( changeDate() ) );
  connect( ebal, SIGNAL( lostFocus() ), SLOT( refresh() ) );
  connect( ebal, SIGNAL( returnPressed() ), SLOT( refresh() ) );
  
  int align=AlignCenter;
  QGridLayout * layout=new QGridLayout( this, 15, 4, 5 );
  QWidget * widgs[]={ new QLabel( tr( "closing date" ), this ), enddate,
											new QLabel( tr( "opening balance" ), this ), lblOBal,
											new QLabel( tr( "closing balance" ), this ), ebal,
											new QLabel( tr( "difference" ), this ), lblDiff,
											ok, cancel };
	
  for( int i=0; i<10; i++ ){
		// the last two widgets should span the whole width
		if( i>7 ) align=0;
		layout->addMultiCellWidget( widgs[i], i, i, 0, 0, align );
  }
  layout->addMultiCellWidget( tView, 0, 14, 1, 3 );

	readPrefs( true );
}

QHaccRecWin::~QHaccRecWin(){}

void QHaccRecWin::setAccount( const Account& a ){
	account=a;
	changeDate();
	tView->setAccount( account );
	tView->setLedger( ledger );
	setCaption( tr( "Reconcile" )+" "+ledger.gets( QHacc::LNAME )+"/"+
							account.gets( QHacc::ANAME ) );
}

void QHaccRecWin::readPrefs( bool initial ){
	tView->readPrefs( initial );
	if( initial ){
		connect( engine, SIGNAL( changedP( const QString&, QString ) ),
						 this, SLOT( changeP( const QString&, QString ) ) );
		connect( engine, SIGNAL( changedP( const QString&, int ) ),
						 this, SLOT( changeP( const QString&, int ) ) );
		connect( engine, SIGNAL( updatedT( const Transaction& ) ),
						 this, SLOT( updateT( const Transaction& ) ) );
		connect( engine, SIGNAL( addedT( const Transaction& ) ),
						 this, SLOT( updateT( const Transaction& ) ) );
		connect( engine, SIGNAL( removedA( const Account& ) ),
						 this, SLOT( removeA( const Account& ) ) );
	}

	QString str=engine->getSP( "RECWINSIZE" );
	if( !str.isEmpty() ){
		QString vals[2];
		Utils::parser( str, " ", 0, vals, 2 );
		resize( vals[0].toInt(), vals[1].toInt() );
	}
}

void QHaccRecWin::save(){
	QString str;
	QSize s=size();
	engine->setSP( "RECWINSIZE",	str.setNum( s.width() )+
								 " "+str.setNum( s.height() ) );
	tView->save();
}

void QHaccRecWin::changeP( const QString& str, QString ){
	if( str=="DATESEPARATOR" ) enddate->refresh();
	else if( str=="CSYMBOL" ) refresh();
}

void QHaccRecWin::changeP( const QString& str, int ){
	if( str=="DATEFORMAT" )	enddate->refresh();
}

void QHaccRecWin::changeDate() {
	// first, make sure we don't have any
	// intermediately-reconciled transaction out there
	
	QDate date=enddate->getDate();
 	closer( false, date );

	// update the view with the correct date
	TableSelect criteria[]={
		TableSelect( QHacc::TRECO, TableCol( QHacc::YREC ), TableSelect::NE ),
		TableSelect( QHacc::TDATE, TableCol( date ), TableSelect::LE )
	};
	tView->setSelectors( criteria, 2 );
	tView->setLimit( date );
	refresh();
}

void QHaccRecWin::refresh(){
	QDate end=enddate->getDate();
	QHaccTable * trans=0;
	TableSelect criteria[]={
		TableSelect( QHacc::TACCTID, account.get( QHacc::AID ) ),
		TableSelect( QHacc::TDATE, TableCol( end ), TableSelect::LE ),
		TableSelect( QHacc::TRECO, TableCol( QHacc::YREC ), TableSelect::NE ),
	};
	engine->getTWhere( criteria, 3, trans );
	
	int balforward=engine->getARBalTo( account, end );
	int uendbal=engine->convMoney( ebal->text() ); // user end bal
	int recd=0;
 
	for( uint i=0; i<trans->rows(); i++ ){
		Transaction t=trans->at( i );
		if( !t.getu( QHacc::TRECO )==QHacc::NREC )
			recd+=engine->convMoney( t.gets( QHacc::TSUM ) );
	}

	/*
	cout<<"recwin refresh:"<<endl;
	cout<<"  balforward date: "<<TableCol( start ).toString()<<endl;
	cout<<"  balance forward: "<<balforward<<endl;
	cout<<"  ending balance:  "<<uendbal<<endl;
	cout<<"  reconciled:      "<<recd<<endl;
	cout<<"  difference:      "<<uendbal-balforward-recd<<endl;
	*/

	lblOBal->setText( engine->convMoney( balforward ) );
	lblDiff->setText( engine->convMoney( uendbal-balforward-recd ) );

	delete trans;
}

void QHaccRecWin::closer( bool rec, const QDate& date ){
	QHaccTable * trans=0;
	tView->getContents( trans );

	if( rec ){ // convert mid-reconcile trans into reconciled
		for( uint i=0; i<trans->rows(); i++ ){
			if( trans->at( i ).getu( QHacc::TRECO )==QHacc::MREC )
				engine->setTRec( trans->at( i ), QHacc::YREC );
		}
	}
	else{ 
		// convert all the transactions to unreconciled, or, if a
		// date is given unreconcile everything after that date

		if( !date.isNull() )
			trans->deleteWhere( TableSelect( QHacc::TDATE, TableCol( date ),
																			 TableSelect::LT ) );
		
		for( uint i=0; i<trans->rows(); i++ )			
			engine->setTRec( trans->at( i ), QHacc::NREC );
	}
	
	delete trans;
}

void QHaccRecWin::accept(){
	disconnect( engine, SIGNAL( updatedT( const Transaction& ) ),
							this, SLOT( updateT( const Transaction& ) ) );
	hide();
	closer( true );
	//save();
	QWidget::close( true );
}

void QHaccRecWin::hide(){
	save();
  QWidget::hide();
}

void QHaccRecWin::cancel(){
	disconnect( engine, SIGNAL( updatedT( const Transaction& ) ),
							this, SLOT( updateT( const Transaction& ) ) );
	hide();
	closer( false );
	//save();
  QWidget::close( true );
}

void QHaccRecWin::updateT( const Transaction& t ){
	if( t.getu( QHacc::TACCTID )==account.getu( QHacc::AID ) )	refresh();
}

void QHaccRecWin::removeA( const Account& a ){
	if( a.compareTo( account, QHacc::AID )==0 ) close();
}
