/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************** * * * * * * * * * * * * **************************/

#include "qhaccutils.h"
#include "qhacctable.h"

#include <qfile.h>
#include <qtextstream.h>

/*****************************************************************/
/* QHACCTABLE                                                    */
/*   This is the basis of all data storage in QHacc. This table  */
/*   will keep an index of ids if necessary, but otherwise       */
/*   provides no search functionality. Use a QHACCTABLEINDEX to  */
/*   index arbitrary columns.                                    */
/*                                                               */
/*                                                               */
/*****************************************************************/

// these are error codes
const int QHaccTable::VALID=0;
const int QHaccTable::COLCOUNT=-1;
const int QHaccTable::COLTYPE= -2;
const int QHaccTable::NULLROW= -3;
const int QHaccTable::DUPLKEY= -4;
const int QHaccTable::NOTFOUND=-5;

const int TableSelect::EQ=0;
const int TableSelect::NE=1;
const int TableSelect::GT=2;
const int TableSelect::LT=3;
const int TableSelect::GE=4;
const int TableSelect::LE=5;
const int TableSelect::UQ=6;
const int TableSelect::NO=7;
const int TableSelect::ALL=8;


const char * QHaccTable::error( int code ){
	static const char * ERRORS[]={ "no error", "column count does not match",
																 "column types do not match", "null row",
																 "column key already exists",
																 "update row not found" };
	if( code>0 ) code=0;
	return ERRORS[-code];
}

QHaccTable::QHaccTable(	 const ColType * cs, const char ** ns, int sz,
												 int _pkcol, const char * n, uint init, uint grow,
												 uint max ) {
	
	if( n ){
		name=QString( n );
		std::ostream * str;
		if( debug( 5, str ) )
			*str<<"creating "<<n<<" with init="<<init
					<<"; growby="<<grow<<"; maxfree="<<max<<endl;
	}

	deleter=true;
	loading=false;
	arrSize=init;
  growBy=grow;
  maxFree=max;
  size=0;
	data=new TableRow * [arrSize];

	cols=sz;
	types=new ColType[cols];
	names=new QString[cols];

	for( int i=0; i<cols; i++ ){
		types[i]=cs[i];
		names[i]=ns[i];
	}

	pkcol=_pkcol;
	addIndexOn( pkcol, pki );
	indexes.setAutoDelete( true );
}

QHaccTable::QHaccTable( const QHaccTable& model ){
	arrSize=model.arrSize;
	growBy=model.growBy;
	maxFree=model.maxFree;
	loading=false;
	deleter=true;

	cols=model.cols;
	types=new ColType[cols];
	names=new QString[cols];
	for( int i=0; i<cols; i++ ){
		types[i]=model.types[i];
		names[i]=model.names[i];
	}
	
	size=model.size;
	data=new TableRow * [arrSize];

	//load( model );
	for( uint i=0; i<size; i++ ) data[i]=new TableRow( model.at( i ) );
	for( uint i=size; i<arrSize; i++ ) data[i]=0;
	
	pkcol=model.pkcol;
	addIndexOn( pkcol, pki );
	QHaccTableIndex * junk=0; // we'll need this to fill up args to addindexon
	for( QHaccTableIndex * idx=indexes.first(); idx!=0; idx=indexes.next() ){
		addIndexOn( idx->sorts(), junk );
	}
	indexes.setAutoDelete( true );
}

QHaccTable& QHaccTable::operator=( const QHaccTable& model ){
	if( &model!=this ){
		for( uint i=0; i<size; i++ ) delete data[i];
		delete [] data;

		arrSize=model.arrSize;
		growBy=model.growBy;
		maxFree=model.maxFree;
		loading=false;
		deleter=true;

		cols=model.cols;
		delete [] types;
		delete [] names;
		types=new ColType[cols];
		names=new QString[cols];
		for( int i=0; i<cols; i++ ){
			types[i]=model.types[i];
			names[i]=model.names[i];
		}
		
		indexes.clear();
		size=model.size;
		data=new TableRow * [arrSize];
		
		for( uint i=0; i<size; i++ ) data[i]=new TableRow( model.at( i ) );
		for( uint i=size; i<arrSize; i++ ) data[i]=0;

		delete pki;
		pkcol=model.pkcol;
		addIndexOn( pkcol, pki );
		QHaccTableIndex * junk=0;
		for( QHaccTableIndex * idx=indexes.first(); idx!=0; idx=indexes.next() ){
			addIndexOn( idx->sorts(), junk );
		}

		indexes.setAutoDelete( true );
	}
	return *this;
}

QHaccTable::~QHaccTable(){
	std::ostream * str;
	if( debug( 5, str ) )
		*str<<"destroying "<<name<<" and all its data"<<endl;

	delete [] types;
	delete [] names;

	indexes.clear();

	if( deleter ) for( uint i=0; i<size; i++ ) delete data[i];
	delete [] data;
}
bool QHaccTable::debug( int lvl, std::ostream *& str ) const {
	return !name.isNull() && Utils::debug( lvl, str );
}
bool QHaccTable::error( int lvl, std::ostream *& str ) const {
	return !name.isNull() && Utils::error( lvl, str );
}
void QHaccTable::setAutoDelete( bool b ) { deleter=b; }
void QHaccTable::setName( QString s ){ name=s; }
int QHaccTable::idcol() const { return pkcol; }
void QHaccTable::clear(){
	if( !data )  return;

	for( uint i=0; i<size; i++ ) delete data[i];
	delete [] data;
	
	size=0;
	arrSize=maxFree;

	data=new TableRow * [arrSize];
	for( uint i=size; i<arrSize; i++ ) data[i]=0;
	remake();
}

bool QHaccTable::save( QFile& file, QString& msg ) const {
	bool ret=false;
	std::ostream * str=0;

	if( file.open( IO_WriteOnly ) ){
		QTextStream out( &file );
		for( uint i=0; i<rows(); i++ ) out<<pki->at( i ).toString()<<endl;
		file.close();

		if( debug( 4, str ) )
			*str<<"wrote "<<rows()<<" rows to "<<name<<" to "<<file.name()<<endl;

		ret=true;
	}
	else{
		msg="could not write "+name+" to "+file.name();
		if( error( 2, str ) ) *str<<msg<<endl;
		ret=false;
	}

	return ret;
}

bool QHaccTable::load( QFile& file, QString& err ){
	std::ostream * str=0;
	if( file.exists() && file.open( IO_ReadOnly ) ){
		QTextStream in( &file );
		int count=0;

		// first, figure out how many rows we're going to load
		while( !in.atEnd() ) {
			in.readLine();
			count++;
		}
		
		// get back to the beginning of the file and start loading
		file.at( 0 );
		startLoad( count );
		while ( !in.eof() )	loadRow( in.readLine() );
		stopLoad();

		file.close();
		if( debug( 4, str ) )
			*str<<"loaded "<<count<<" rows from "<<file.name()<<" into "<<name<<endl;
	}
	else{
		err="could not open file: "+file.name();
		if( error( 2, str ) ) *str<<err<<endl;
		return false;
	}
	return true;
}

bool QHaccTable::load( const QHaccTable& donor ){
	// load copies of all the rows in donor table
	uint drows=donor.rows();
	startLoad( drows );
	for( uint i=0; i<drows; i++ )	add( donor.at( i ) );
	stopLoad();
	
	return true; // always works?
}

void QHaccTable::startLoad( uint rows ){
	std::ostream * str=0;
	if( debug( 5, str ) )
		*str<<"starting load of "<<rows<<" rows into "<<name<<endl;
	loading=true;
	if( rows+size>arrSize ) resizeTo( arrSize+rows );
}

void QHaccTable::stopLoad(){
	loading=false;
	if( arrSize-size>maxFree ) resizeTo( size+( maxFree/2 ) );
	remake();

	std::ostream * str;
	if( debug( 5, str ) )	*str<<"ending load of "<<name<<endl;
}

void QHaccTable::loadRow( const QString& str ) {
	QString strs[cols];
	QString * tmp=0;

	Utils::parser( str, "||", 0, strs, cols );
	TableCol tc[cols];

	for( int i=0; i<cols; i++ ){
		switch( types[i] ){
		case CTFLOAT:
			tc[i]=TableCol( strs[i].toFloat() );
			break;
		case CTINT:
			tc[i]=TableCol( strs[i].toInt() );
			break;
		case CTUINT:
			tc[i]=TableCol( strs[i].toUInt() );
			break;
		case CTDATE:
			tmp=new QString[3];
			Utils::parser( strs[i], "/", 0, tmp, 3 );
			tc[i]=TableCol( QDate( tmp[2].toInt(), tmp[0].toInt(),
														 tmp[1].toInt() ) );
			delete [] tmp;
			break;
		case CTBOOL:
			tc[i]=TableCol( strs[i]=="Y" ? true : false );
			break;
		default:
			tc[i]=TableCol( strs[i] );
		}
	}
	
	TableRow * row=new TableRow( tc, cols );
	int valid=verifyRow( row );
	if( valid<0 ){
		delete row;
		return;
	}
	
	if( size==arrSize ) resizeTo( arrSize+growBy );
	data[size++]=row;
}

bool QHaccTable::isEmpty() const { return size==0; }
uint QHaccTable::rows() const { return size; }
uint QHaccTable::columns() const { return cols; }
QString QHaccTable::colname( int i ) const { return names[i]; }
int QHaccTable::verifyRow( const TableRow * prow ) const {
	// verify that the given row has the same number/type fo columns
	int reason=VALID;
	if( !prow ) reason=NULLROW;
	else{
		const TableRow& row=*prow;

		if( row.size!=cols ) reason=COLCOUNT;
		else{
			for( int i=0; i<cols; i++ ){
				if( row.gett( i )!=types[i] ) reason=COLTYPE;
				if( row.gett( i )!=types[i] ) cerr<<"col "<<i<<" is type "<<row.gett( i )<<" but it should be "<<types[i]<<endl;
			}
		}
	}

	std::ostream * str=0;
	if( reason<0 && error( 2, str ) ){
		*str<<"INVALID: "<<error( reason )<<endl
				<<"\t"<<prow->toString()<<endl;
	}

	return reason;
}

bool QHaccTable::contains( const TableCol& col, uint& idx ) const {
	// we're only searching for pks, and luckily, we have and index for that. 
	idx=0;
	bool inthere=pki->contains( col, idx );

	// convert the index's idx to the table's idx
	idx=pki->loc( idx );
	return inthere;
}

bool QHaccTable::contains( const TableRow& model, uint& idx ) const {
	
	if( pkcol==-1 ){
		for( uint i=0; i<rows(); i++ ){
			bool found=true;
			for( int j=0; j<cols; j++ )
				found=( found && ( model[j].compareTo( data[i]->get( j ) )==0 ) );
			
			if( found ){
				idx=i;
				return true;
			}
		}
		return false;
		//std::ostream * str=0;
		//if( error( 2, str ) )
		//*str<<"cannot call qhacctable::contains() without a primary key"<<endl;
		//return false;
	}
	else return contains( model.get( pkcol ), idx );
}

void QHaccTable::reindex(){
	if( !loading ){
		//pki->reindex();
		for( QHaccTableIndex * idx=indexes.first(); idx!=0; idx=indexes.next() )
			idx->reindex();
	}
}

void QHaccTable::remake(){
	//pki->remake();

	for( QHaccTableIndex * idx=indexes.first(); idx!=0; idx=indexes.next() )
		idx->remake();
}

int QHaccTable::add( const TableRow& row ){	return add( new TableRow( row ) );}
int QHaccTable::add( TableRow * row ){
	// add a new row...first verify that the row is valid
	int valid=verifyRow( row );
	if( valid<0 ) return valid;

	std::ostream * str=0;

	// then screen for duplicate keys if we're not loading
	uint junk=0;
	if( !loading ){
		if( pkcol!=-1 && contains( *row, junk ) ){
			if( error( 2, str ) ){
				TableRow * old=0;
				at( junk, old );
				*str<<"INVALID: "<<error( DUPLKEY )<<endl
						<<"\t"<<row->toString()<<"  conflicts with "<<endl
						<<"\t"<<old->toString()<<endl;
			}
			
			return DUPLKEY;
		}
	}

	if( debug( 6, str ) )
		*str<<"adding row to "<<name<<": "<<row->toString()<<endl;

	if( size==arrSize )	resizeTo( arrSize+growBy );
	data[size++]=row;
	if( !loading ) reindex();
	
	return VALID;
}

void QHaccTable::dropIndex( QHaccTableIndex * idx ){
	indexes.removeRef( idx );
}

bool QHaccTable::getIndexOn( int pos, int spos, QHaccTableIndex *& ret ){
	// get an index for pos if it exists
	// return true if we find an index
	ret=0;
	//if( pki->sorts( pos, spos ) ) ret=pki;
	//else{
	for( QHaccTableIndex * idx=indexes.first(); idx!=0; idx=indexes.next() )
		if( idx->sorts( pos, spos ) ) ret=idx;	
		//}
	return ret!=0;
}

bool QHaccTable::getIndexOn( int pos, QHaccTableIndex *& ret ){
	// get an index for pos if it exists
	// return true if we find an index
	ret=0;
	//if( pos==pkcol ) ret=pki;
	//else{
	for( QHaccTableIndex * idx=indexes.first(); idx!=0; idx=indexes.next() )
		if( idx->sorts( pos ) ) ret=idx;	
	//}
	return ret!=0;
}

void QHaccTable::igetWhere( const TableSelect& sel, QHaccTable * table,
														QHaccTable *& ret ){
	const char * ns[table->cols];
	for( int i=0; i<table->cols; i++ ) ns[i]=table->names[i].latin1();
	
	ret=new QHaccTable( table->types, ns, table->cols, table->pkcol );
	ret->setAutoDelete( false );

	int col, check;
	TableCol model;
	sel.getAll( col, model, check );
	//if( name=="ryan" ) cout<<"igw "<<sel.toString()<<endl;
	if( check==TableSelect::ALL ) ret->load( *table );
	else if( check!=TableSelect::NO ){
		QHaccTableIndex * useme=0;
		if( table->getIndexOn( col, useme ) ){ // USE INDEX
			//if( name=="ryan" ) cout<<"  igw using an index"<<endl;
			uint s=useme->starts( model );
			uint e=useme->ends( model );

			if( check==TableSelect::NE ){
				for( uint i=0; i<s; i++ ){
					TableRow * row=0;
					table->at( useme->loc( i ), row );
					ret->add( row );
				}
				for( uint i=e; i<table->rows(); i++ ){
					TableRow * row=0;
					table->at( useme->loc( i ), row );
					ret->add( row );
				}
			}
			else if( check==TableSelect::UQ ) useme->unique( ret );
			else{
				uint start=0, end=table->rows();
					
				if( check==TableSelect::EQ ){
					start=s;
					end=e;
				}
				
				else if( check==TableSelect::GT )	start=e;
				else if( check==TableSelect::LT )	end=s;
				else if( check==TableSelect::GE ) start=s;
				else if( check==TableSelect::LE ) end=e;
				
				//if( name=="ryan" )cout<<"    idx says "<<s<<"/"<<e<<", but logic says "<<start<<"/"<<end<<endl;
				
				ret->startLoad( end-start );
				for( uint i=start; i<end; i++ ){
					TableRow * row=0;
					table->at( useme->loc( i ), row );
					ret->add( row );				
					//if( name=="ryan" ) cout<<"\tadding "<<row->toString()<<endl;
				}
			}
		}
		else{ // NO INDEX
			// we need to check every row for matches SLOW!
			//if( name=="ryan" ) cout<<"  igw NOT using an index"<<endl;
			ret->startLoad();

			if( check==TableSelect::UQ ){
				// figuring out uniqueness on an unindexed table
				// would be exceedingly slow...it's better just
				// to make an index
				QHaccTableIndex * junk=0;
				table->addIndexOn( col, junk );
				igetWhere( sel, table, ret );
			}
			else{
				for( uint i=0; i<table->rows(); i++ ){
					TableRow * row=0;
					table->at( i, row );
					if( sel.check( *row ) ) ret->add( row );
				}
			}
		}
		ret->stopLoad();
	}

	// if we used an index to get the data, put
	// an index on the same column going forward
	QHaccTableIndex * junk=0;
	ret->addIndexOn( col, junk );

	/*
	if( name=="ryan" ){
		cout<<"result is:"<<endl;
		for( uint i=0; i<ret->rows(); i++ )
			cout<<"  "<<ret->at( i ).toString()<<endl;
	}
	*/
}

bool QHaccTable::getWhere( TableSelect * arr, uint sz, QHaccTable *& ret ){
	if( sz>0 ){
		QHaccTable * temp=0;
		igetWhere( arr[0], this, temp );
		//if( name=="ryan" ) temp->setName( "ryan" );
		
		for( uint i=1; i<sz; i++ ){
			QHaccTable * bobo=0;
			igetWhere( arr[i], temp, bobo );
			//if( name=="ryan" ) bobo->setName( "ryan" );
			delete temp;
			temp=bobo;
		}
	
		ret=new QHaccTable( *temp );
		delete temp;

		// make sure we have all our necessary indicies
		QHaccTableIndex * junk=0; // we'll need this to fill up args to addindexon
		for( QHaccTableIndex * idx=indexes.first(); idx!=0; idx=indexes.next() ){
			ret->addIndexOn( idx->sorts(), junk );
		}
	}
	else ret=new QHaccTable( *this );
	
	return !ret->isEmpty();
}

bool QHaccTable::getWhere( const TableSelect& sel, QHaccTable *& ret ){
	QHaccTable * temp=0;
	igetWhere( sel, this, temp );
	ret=new QHaccTable( *temp );
	delete temp;

	// make sure all the indicies are created so the return table looks
	// just like this table
	QHaccTableIndex * junk=0; // we'll need this to fill up args to addindexon
	for( QHaccTableIndex * idx=indexes.first(); idx!=0; idx=indexes.next() ){
		ret->addIndexOn( idx->sorts(), junk );
	}
	
	return !ret->isEmpty();
}

uint QHaccTable::deleteWhere( int col, const TableCol& model ){
	// delete all rows that match model in the col column
	// hopefully, this function avoids unecessary repaints
	// by creating an array of indexes to delete, then 
	// passing the array to deleteWhere( idx, count );
	uint deletes=0;
	
	QHaccTableIndex * user=0;
	if( getIndexOn( col, user ) ){
		uint start=user->starts( model );
		uint end=user->ends( model );
		
		if( end>start ){
			deletes=end-start;
			if( deletes>0 ){
				uint indexes[deletes];
				for( uint i=0; i<deletes; i++ )	indexes[i]=user->loc( i+start );
				deleteWhere( indexes, deletes );
			}
		}
	}
	else{
		// without an index, we need to do a full table scan
		uint rws=rows();
		uint indexes[rws];

		for( uint i=0; i<rws; i++ )
			if( data[i]->compareTo( model, col )==0 )	indexes[deletes++]=i;

		deleteWhere( indexes, deletes );
	}

	return deletes;
}

uint QHaccTable::deleteWhere( const TableSelect& criteria ){
	// just like the other deleteWheres, only using a TableSelect
	// we try an optimization: if the TableSelect specifies ALL,
	// we'll pass the data on to deleteWhere( int, col ), where, if 
	// we have an index, things go lickety-split. otherwise, go the
	// slower route of deleteWhere( Table )

	TableCol model;
	int pos;
	int check;
	criteria.getAll( pos, model, check );

	uint dels=0;
	if( check==TableSelect::ALL )	dels=deleteWhere( pos, model );
	else{
		QHaccTable * temp=0;
		getWhere( criteria, temp );
		dels=deleteWhere( temp );
		delete temp;
	}

	return dels;
}

uint QHaccTable::deleteWhere( QHaccTable * dels ){
	// delete everything in the the dels table. this function hopes to
	// reduce reindexing when deleting potentially lots of rows. we'll
	// do this by making an array of locations to delete first and then
	// deleting everything in one fell swoop!

	uint rws=dels->rows();
	uint indexes[rws];

	uint idx=0, count=0;
	for( uint i=0; i<rws; i++ ){
		TableRow * row=0;
		dels->at( i, row );
		if( contains( *row, idx ) ) indexes[count++]=idx;
	}

	deleteWhere( indexes, count );
	return count;
}

void QHaccTable::deleteWhere( uint * indexes, uint size ){
	startLoad();
	for( uint i=0; i<size; i++ ){
		remove( indexes[i] );
		// removal changes the locations
		// of rows, so make adjustments
		for( uint j=i+1; j<size; j++ ) if( indexes[j]>indexes[i] )	indexes[j]--;
	}
	stopLoad();	
}

void QHaccTable::remove( uint idx ){
	std::ostream * str=0;
	if( debug( 6, str ) ) *str<<"removing row from "<<name<<": "
														<<data[idx]->toString()<<endl;

	TableRow * row=data[idx];
	for( uint i=idx; i<size-1; i++ ) data[i]=data[i+1];

	delete row;
	data[--size]=0; // don't leave junk laying around

	if( !loading ){
		if( ( arrSize-size )>maxFree ) resizeTo( size+maxFree );
		else reindex();
	}
}

void QHaccTable::resizeTo( uint newsize ){
	std::ostream * str=0;
	if( debug( 5, str ) ) *str<<"resizing "<<name<<": "
														<<arrSize<<"->"<<newsize<<endl;

	arrSize=newsize;

	TableRow ** newdata=new TableRow * [newsize];
	
	for( uint i=0; i<size; i++ ) newdata[i]=data[i];

	delete [] data;
	data=newdata;
  remake();
}

void QHaccTable::at( uint i, TableRow *& ret ){	ret=data[i]; }
TableRow QHaccTable::at( uint i ) const { return TableRow( *data[i] ); }

void QHaccTable::addIndexOn( int col, QHaccTableIndex *& ret ){
	// create and index on a column;
	if( !getIndexOn( col, ret ) ){
		ret=new QHaccTableIndex( this, col );
		indexes.append( ret );
  }
}

void QHaccTable::addIndexOn( int col, int subcol, QHaccTableIndex *& ret ){
	// create and index on a column;
	bool haspri=getIndexOn( col, ret );
	if( ( haspri && !ret->sorts( col, subcol ) ) || !haspri ){
		ret=new QHaccTableIndex( this, col, subcol );
		indexes.append( ret );
	}
}

TableCol QHaccTable::maxID() const { return pki->maxID(); }
TableCol QHaccTable::minID() const { return pki->minID(); }

void QHaccTable::update( const TableRow * old, const TableRow& row ) {
	uint idx=0;
	if( contains( *old, idx ) ){
		//if( name=="ryan" ) cout<<"updating: "<<data[idx]->toString()<<endl<<"...to look like: "<<row.toString()<<endl;
		delete data[idx];
		data[idx]=new TableRow( row );
	}
}

int QHaccTable::updateWhere( int pos, const TableCol& equal,
														 const TableRow& row ){
	int valid=verifyRow( &row );
	if( valid<0 ) return valid;

	QHaccTable * goodRows=0;
	igetWhere( TableSelect( pos, equal ), this, goodRows );
	
	uint rws=goodRows->rows();

	// keep track of which positions we're actually
	// changing, so we can reindex only as needed
	bool reindexes[cols];
	for( int i=0; i<cols; i++ ) reindexes[i]=false;

	for( uint i=0; i<rws; i++ ){
		TableRow * old=0;
		goodRows->at( i, old );
		for( int j=0; j<cols; j++ ){
			if( old->get( j ).compareTo( row.get( j ) )!=0 ) reindexes[i]=true;
		}
		update( old, row );
	}
	delete goodRows;

	if( rws ){
		for( int i=0; i<cols; i++ ){
			if( reindexes[i] ){
				QHaccTableIndex * idx=0;
				if( getIndexOn( i, idx ) ) idx->reindex();
			}
		}
		return VALID;
	}
	else return NOTFOUND;
}

int QHaccTable::updateWhere( const TableSelect& criteria, 
														 int pos, const TableCol& replace ){
	QHaccTable * goodRows=0;
	igetWhere( criteria, this, goodRows );
	uint rws=goodRows->rows();

	for( uint i=0; i<rws; i++ ){
		TableRow * old=0;
		goodRows->at( i, old );
		TableRow row( *old );
		row.set( pos, replace );
		update( old, row );
	}
	delete goodRows;

	if( rws ){
		// only reindex what we have to
		QHaccTableIndex * idx=0;
		if( getIndexOn( pos, idx ) ) idx->reindex();
		return VALID;	
	}
	else return NOTFOUND;	
}

int QHaccTable::updateWhere( int pos, const TableCol& equal,
														 int cpos, const TableCol& ncol ){
	return updateWhere( TableSelect( pos, equal ), cpos, ncol );
}



/***********************************************************/
/* QHACCTABLEINDEX                                         */
/*   an index for a column in a QHACCTABLE. This works on  */
/*   any type of column, and uses a quicksort to provide   */
/*   the ordering. It does not automatically refresh       */
/*   itself when the base table changes, so be very        */
/*   careful to call reindex() when that has happened.     */
/*                                                         */
/***********************************************************/

QHaccTableIndex::QHaccTableIndex( QHaccTable * table, int f ){
	init( table, f, -1 );
	if( table ) reindex();
}

QHaccTableIndex::QHaccTableIndex( QHaccTable * table, int f, int s ){
	init( table, f, s );
	if( table ) reindex();
}

QHaccTableIndex::QHaccTableIndex( const QHaccTableIndex& model ){
	data=model.data;
	field=model.field;
	lookup=new uint[data->arrSize];
	for( uint i=0; i<data->rows(); i++ ) lookup[i]=model.lookup[i];
}

QHaccTableIndex& QHaccTableIndex::operator=( const QHaccTableIndex& model ){
	if( &model!=this ){
		data=model.data;
		field=model.field;

		delete [] lookup;
		lookup=new uint[data->arrSize];
		for( uint i=0; i<data->rows(); i++ ) lookup[i]=model.lookup[i];
	}
	return *this;
}

void QHaccTableIndex::init( QHaccTable * table, int f1, int f2 ){
	data=table;

	uint rows=0;
	if( data ) rows=data->arrSize;

	lookup=new uint[rows>0 ? rows : 1];
	for( uint i=0; i<rows; i++ ) lookup[i]=i;
	field=f1;
	subfield=f2;
}

QHaccTableIndex::~QHaccTableIndex(){ delete [] lookup; }

int QHaccTableIndex::sorts() const { return field; }
bool QHaccTableIndex::sorts( int i ) const {
	if( field==-1 ) return false;
	return( i==field );
}
bool QHaccTableIndex::sorts( int i, int j ) const { 
	if( field==-1 ) return false;
  return( i==field && j==subfield );
}

uint QHaccTableIndex::loc( uint idx ) const { return lookup[idx]; }
TableRow QHaccTableIndex::at( uint i ) const { return data->at( loc( i ) ); }
void QHaccTableIndex::at( uint i, TableRow *& ret ) { 
	return data->at( loc( i ), ret ); 
}

void QHaccTableIndex::remake(){
	uint sizer=data->arrSize;
	delete [] lookup;
	lookup=new uint[sizer];
	reindex();
}

void QHaccTableIndex::reindex(){
	uint rows=data->rows();

	std::ostream * str=0;
	if( data->debug( 6, str ) ){
		*str<<"reindex called on "<<data->name<<"::"
				<<( field==-1 ? QString( "NONE" ) : data->names[field] )<<endl;
	}

	if( data->isEmpty() ) return;


	for( uint i=0; i<data->arrSize; i++ ) lookup[i]=i;

	if( field>-1 ) qsort( 0, rows-1 );
}

void QHaccTableIndex::qsort( uint first, uint last ){
	if( first<last ){
		uint q=findQ( first, last );
		qsort( first, q );
		qsort( q+1, last );
	}
}

uint QHaccTableIndex::findQ( uint p, uint r ) {
	uint pivot=( p+r )/2;
	TableRow * model=0;
	at( pivot, model );
	TableRow * check=0;

	uint i=p-1;
	uint j=r+1;

	if( subfield==-1 ){
		while( true ) {
			do {
				j--;
				at( j, check );
			} while( check->compareTo( *model, field )>0 );
			
			do {
				i++;
				at( i, check );
			} while( check->compareTo( *model, field )<0 );
			
			if( i<j )	swap( i, j );
			else return j;
		}
	}
	else{
		while( true ) {
			do {
				j--;
				at( j, check );
			} while( check->compareTo( *model, field, subfield )>0 );
			
			do {
				i++;
				at( i, check );
			} while( check->compareTo( *model, field, subfield )<0 );
			
			if( i<j )	swap( i, j );
			else return j;
		}
	}
}

void QHaccTableIndex::swap( uint i, uint j ){
	uint temp=lookup[i];
	lookup[i]=lookup[j];
	lookup[j]=temp;
}

uint QHaccTableIndex::ifind( const TableCol& col, uint start, uint end,
														 bool& found ){
	bool done=found=false;
	uint cidx=0;

	//if( data->name=="ryan" ){cout<<"  data is: "<<endl;for( uint i=0; i<rows(); i++ ) cout<<"    "<<i<<":\t"<<at( i ).toString()<<endl;cout<<"  END"<<endl;}

	while( !done ){
		cidx=( start+end )/2;
		TableRow * check=0;
		at( cidx, check );

		
		//if( data->name=="ryan" ){cout<<"    ifind "<<start<<"->"<<end<<"="<<cidx<<endl;cout<<"\t"<<check->toString()<<endl;}

		int result=check->compareTo( col, field );
		if( result==0 )	found=done=true;
		else{
			if( result>0 ) end=cidx;
			else start=cidx+1;
		}
		
		if( !done && start>=end ) done=true;
	}

	return cidx;
}

uint QHaccTableIndex::starts( const TableCol& col ) {
	// return the position where the given TableCol
	// starts. if a perfect match can't be found, 
	// return the position where the col would be inserted
	if( data->isEmpty() || field==-1 ) return 0;

	//if( data->name=="ryan" ) cout<<"  index looking for start of "<<col.toString()<<" in col "<<field<<endl;

	bool found=false;
	uint idx=ifind( col, 0, data->rows(), found );
	if( found ){
		// we don't have an idea of unique indexes,
		// so now we need to find the actual start

		//if( data->name=="ryan" ) cout<<"ifind returned "<<idx<<endl;
		uint end=idx;
		idx=0;
		while( idx<end ){
			TableRow * check=0;
			at( idx, check );
			if( check->compareTo( col, field )<0 ) idx++;
			else break;
		}
	}
	else{
		TableRow * row=0;
		at( idx, row );
		if( row->compareTo( col, field )<0 ) idx++;
	}

	//if( data->name=="ryan" ) cout<<"start determined to be "<<idx<<endl;
	return idx;
}	

uint QHaccTableIndex::ends( const TableCol& col ) {
	// find the end position of the given TableCol. 
	// if a perfect match can't be found, return 
	// the position before the first position that is
	// greater than the given TableCol

	if( data->isEmpty() || field==-1 ) return rows();

	//if( data->name=="ryan" ) cout<<"looking for end of "<<col.toString()<<" in col "<<field<<endl;

	bool found=false;
	uint idx=ifind( col, 0, data->rows(), found );
	if( found ){
		// we don't have an idea of unique indexes,
		// so now we need to find the actual start

		//if( data->name=="ryan" ) cout<<"ifind returned "<<idx<<endl;

		uint end=data->rows();
		while( idx<end ){
			TableRow * check=0;
			at( idx, check );
			if( check->compareTo( col, field )==0 ) idx++;
			else break;
		}
	}

	if( idx<rows() ){ // make sure we really have the last index
		TableRow * row=0;
		at( idx, row );
		if( row->compareTo( col, field )<=0 ) idx++;
	}

	//if( data->name=="ryan" ) cout<<"end determined to be "<<idx<<endl;
	
	return idx;
}

bool QHaccTableIndex::contains( const TableCol& col ) {
	uint junk=0;
	return contains( col, junk );
}

bool QHaccTableIndex::contains( const TableRow& row, uint& idx ) {
	if( field==-1 || data->isEmpty() ) return false;
	TableCol col=row.get( field );

	if( subfield>=0 ){
		idx=starts( col );
		uint ender=ends( col );
		
		for( ; idx<ender; idx++ ){
			TableRow * check=0;
			data->at( loc( idx ), check );
			if( row.compareTo( *check, field, subfield )==0 ) return true;
		}
		return false;
	}
	else return contains( col, idx );
}

bool QHaccTableIndex::contains( const TableCol& col, uint& idx ) {
	// if we're not keeping an index, we can't tell what we have,
	// but we probably don't care about duplicates, so it doesn't matter
	if( field==-1 || data->isEmpty() ) return false;

	idx=starts( col );
	if( idx<data->rows() ) {
		TableRow * row=0;
		data->at( loc( idx ), row );
		return ( row->compareTo( col, field )==0 );
	}
	return false;
}

void QHaccTableIndex::unique( QHaccTable *& ret ){
	uint curloc=0;
	uint end=rows();
	
	TableRow * row=0;
	while( curloc<end ){
		at( curloc, row );
		ret->add( row );
		curloc=ends( row->get( field ) );
	}
}

TableCol QHaccTableIndex::maxID() {
	if( field==-1 || data->isEmpty() ) return TableCol( ( uint )0 );
	TableRow * row=0;
	data->at( loc( rows()-1 ), row );
	return row->get( field );
}

TableCol QHaccTableIndex::minID() {
	if( field==-1 || data->isEmpty() ) return TableCol( ( uint )0 );
	TableRow * row=0;
	data->at( loc( 0 ), row );
	return row->get( field );
}

uint QHaccTableIndex::rows() const { return data->rows(); }


/***********************************************************/
/* TABLESELECT                                             */
/*   a TableSelect is one criteria for selecting from      */
/*   tables. QHaccTable accepts an array of them to make   */
/*   complicated selections on the data in the table       */
/*                                                         */
/***********************************************************/

TableSelect::TableSelect( const TableSelect& other ){
	other.getAll( column, model, checktype );
}
TableSelect::TableSelect(){
	column=-1;
	model=TableCol();
	checktype=ALL;
}
TableSelect::TableSelect( int col, TableCol check, int ctype ){
	column=col;
	model=check;
	checktype=ctype;
}
const TableSelect& TableSelect::operator=( const TableSelect& other ){
	if( &other!=this ) other.getAll( column, model, checktype );
	return *this;
}

TableSelect::~TableSelect(){}
void TableSelect::getAll( int& c, TableCol& col, int& t ) const {
	c=column;
	col=model;
	t=checktype;
}

QString TableSelect::toString() const {
	QString temp=QString().setNum( column );
	QString ret="selector for: "+temp+" "+model.toString()+" ";
	
	if( checktype==ALL ) temp="ALL";
	else if( checktype==EQ ) temp="EQ";
	else if( checktype==NE ) temp="NE";
	else if( checktype==GT ) temp="GT";
	else if( checktype==GE ) temp="GE";
	else if( checktype==LT ) temp="LT";
	else if( checktype==LE ) temp="LE";
	else if( checktype==UQ ) temp="UQ";
	else temp="NO";
	return ret+=temp;
}

QString TableSelect::sqlValue() const {
	QString temp;

	if( checktype==EQ ) temp="=";
	else if( checktype==NE ) temp="!=";
	else if( checktype==GT ) temp=">";
	else if( checktype==GE ) temp=">=";
	else if( checktype==LT ) temp="<";
	else if( checktype==LE ) temp="<=";
	else{
		if( checktype==UQ ) temp="UQ";
		else if( checktype==ALL ) temp="ALL";
		else temp="NO";
		return temp+" is not a SQL selector";
	}
	return temp+model.sqlValue();
}

bool TableSelect::check( const TableRow& row ) const {
	if( checktype==ALL ) return true;
	else if ( checktype==NO ) return false;

	int result=row.compareTo( model, column );
	if( result==0 )	return ( checktype==EQ || checktype==GE || checktype==LE );
	else if( result<0 ) return ( checktype==NE || checktype==LT ||
															 checktype==LE || checktype==UQ );
	else return ( checktype==NE || checktype==GT || checktype==GE ||
								checktype==UQ );
}
