/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************** * * * * * * * * * * * * **************************/

#include "qhacctablerows.h"

#include <qregexp.h>

/****************************************/
/* TABLEROW                             */
/*   A row of data in a qhacctable      */
/*                                      */
/****************************************/

TableRow::TableRow(){
	size=0;
	cols=new TableCol[0];
}

TableRow::TableRow( const TableRow& model ){
	size=model.size;

	cols=new TableCol[size];
	for( int i=0; i<size; i++ ) cols[i]=model.cols[i];
}

TableRow::~TableRow(){ delete [] cols; }

TableRow::TableRow( TableCol * acols, int ncols ){
	size=ncols;
	cols=new TableCol[ncols];
	for( int i=0; i<ncols; i++ ) cols[i]=acols[i];
}

QString TableRow::toString() const {
	QString ret;
	for( int j=0; j<size; j++ ) {
		ret+=cols[j].toString();
		if( j+1!=size ) ret+="||";
	}
	return ret;
}

const TableCol& TableRow::operator[]( int i ) const { return cols[i]; }

const TableRow& TableRow::operator=( const TableRow& model ) {
	if( &model!=this ){
		if( cols ) delete [] cols;
		
		size=model.size;
		cols=new TableCol[size];

		for( int i=0; i<size; i++ ) cols[i]=model.cols[i];
	}
	
	return *this;
}
bool TableRow::isNull() const { return size==0; }
TableCol TableRow::get( int i ) const { return cols[i]; }
int TableRow::geti( int i ) const { return cols[i].geti(); }
uint TableRow::getu( int i ) const { return cols[i].getu(); }
float TableRow::getf( int i ) const { return cols[i].getf(); }
bool TableRow::getb( int i ) const { return cols[i].getb(); }
QDate TableRow::getd( int i ) const { return cols[i].getd(); }
QString TableRow::gets( int i ) const { return cols[i].gets(); }
ColType TableRow::gett( int i ) const { return cols[i].gett(); }
bool TableRow::set( int i, TableCol tc ){
	if( tc.gett()!=cols[i].gett() ) return false;
	cols[i]=tc;
	return true;
}

int TableRow::compareTo( const TableCol& model, int i ) const {
	// -1 is <
	//  0 is ==
	//  1 is >
	// nulls always sort to the front
	if( isNull() ) return -1;
	return cols[i].compareTo( model );
}

int TableRow::compareTo( const TableRow& model, int i ) const {
	if( isNull() ) return -1;
	else if( model.isNull() ) return 1;
	
	return compareTo( model[i], i );
}

int TableRow::compareTo( const TableRow& model, int i, int j ) const {
	if( isNull() ) return -1;
	else if( model.isNull() ) return 1;
	
	int result=compareTo( model[i], i );

	// if we get a tie, check the subfield
	if( result==0 )	result=compareTo( model[j], j );
	return result;
}



/****************************************/
/* TABLECOL                             */
/*   One piece of data in a tablerow    */
/*                                      */
/****************************************/

TableCol::TableCol(){ setType( CTNULL ); }

TableCol::TableCol( const char * s ){
	setType( CTSTRING );
	sdata=QString( s );
}
TableCol::TableCol( const QString& s ){
	setType( CTSTRING );
	sdata=QString( s );
}

TableCol::TableCol( int i ){
	setType( CTINT );
	idata=i;
}

TableCol::TableCol( uint i ){
	setType( CTUINT );
	udata=i;
}

TableCol::TableCol( float f ){
	setType( CTFLOAT );
	fdata=f;
}

TableCol::TableCol( const QDate& d ){
	setType( CTDATE );
	ddata=QDate( d );
}

TableCol::TableCol( bool b ){
	setType( CTBOOL );
	bdata=b;
}

void TableCol::setType( ColType ct ) {
	// set the type, and then select the appropriate 
	// comparison function for future use

	type=ct; 
	if( type==CTSTRING ) comparer=&TableCol::compareString;
	else if( type==CTINT ) comparer=&TableCol::compareInt;
	else if( type==CTUINT ) comparer=&TableCol::compareUInt;
	else if( type==CTDATE ) comparer=&TableCol::compareDate;
	else if( type==CTBOOL ) comparer=&TableCol::compareBool;
	else if( type==CTFLOAT ) comparer=&TableCol::compareFloat;
	else comparer=&TableCol::compareNull;

	idata=0;
	udata=0;
	fdata=0;
	bdata=false;
}

int TableCol::geti() const { return idata; }
uint TableCol::getu() const { return udata; }
bool TableCol::getb() const { return bdata; }
float TableCol::getf() const { return fdata; }
QDate TableCol::getd() const { return ddata; }
ColType TableCol::gett() const { return type; }
QString TableCol::gets() const { return sdata; }

const TableCol& TableCol::operator=( const TableCol& model ) {
	if( &model!=this ){
		idata=model.geti();
		udata=model.getu();
		fdata=model.getf();
		ddata=model.getd();
		sdata=model.gets();
		bdata=model.getb();

		type=model.gett();
		comparer=model.comparer;
	}
	
	return *this;
}

int TableCol::compareTo( const TableCol& model ) const {
	return ( *this.*comparer )( model );
}

int TableCol::compareString( const TableCol& model ) const {
	int val=sdata.compare( model.gets() );
	if( val==0 ) return val;
	if( val<0 ) return -1;
	return 1;
}
int TableCol::compareInt( const TableCol& model ) const {
	int i=model.geti();
	if( idata==i ) return 0;
	if( idata<i ) return -1;
	return 1;
}
int TableCol::compareUInt( const TableCol& model ) const {
	uint i=model.getu();
	if( udata==i ) return 0;
	if( udata<i ) return -1;
	return 1;
}
int TableCol::compareFloat( const TableCol& model ) const {
	float i=model.getf();
	if( fdata==i ) return 0;
	if( fdata<i ) return -1;
	return 1;
}
int TableCol::compareDate( const TableCol& model ) const {
	QDate d=model.getd();
	if( ddata==d ) return 0;
	if( ddata<d ) return -1;
	return 1;
}
int TableCol::compareBool( const TableCol& model ) const {
	bool b=model.getb();
	if( bdata==b ) return 0;
	if( !bdata && b ) return -1;
	return 1;
}
int TableCol::compareNull( const TableCol& ) const { return -1; }

QString TableCol::toString() const {
	QString ret;
	switch( type ){
	case CTSTRING:
		ret=sdata;
		break;
	case CTINT:
		ret.setNum( idata );
		break;
	case CTUINT:
		ret.setNum( udata );
		break;
	case CTFLOAT:
		ret.setNum( fdata );
		break;
	case CTBOOL:
		ret=( bdata ? "Y" : "N" );
		break;
	case CTDATE:
		ret=QString().setNum( ddata.month() )+"/"+
			QString().setNum( ddata.day() )+"/"+
			QString().setNum( ddata.year() );
		break;
	default:
		break;
	}
	return ret;
}

QString TableCol::sqlValue() const {
	QString ret;
	switch( type ){
	case CTSTRING:
		ret=sdata;
		ret.replace( QRegExp( "'" ), "''" );
		
		ret.prepend( "'" );
		ret.append( "'" );
		break;
	case CTINT:
		ret.setNum( idata );
		break;
	case CTUINT:
		ret.setNum( udata );
		break;
	case CTFLOAT:
		ret.setNum( fdata );
		break;
	case CTBOOL:
		ret=( bdata ? "1" : "0" );
		break;
	case CTDATE:
		ret.append( "'" );
		ret.append( QString().setNum( ddata.year() )+"-"+
								QString().setNum( ddata.month() )+"-"+
								QString().setNum( ddata.day() ) );
		ret.append( "'" );
		break;
	default:
		break;
	}
	return ret;
}
