/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************** * * * * * * * * * * * * **************************/

#include "config.h"
#include "qhaccview.h"
#include "qhaccutils.h"
#include "transeditor.h"
#include "qhaccdialogs.h"
#include "transeditor.moc"

#include <qheader.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qcheckbox.h>
#include <qscrollview.h>
#include <qpushbutton.h>

const int TransactionEditor::SACCT=0;
const int TransactionEditor::SSUM =1;
const int TransactionEditor::SCOLS=2;

const ColType TransactionEditor::SCOLTYPES[]={ CTUINT, CTSTRING };
const char * TransactionEditor::SCOLNAMES[]={ "ACCTID", "SUM" };
const char * TransactionEditor::SPLIT=QT_TR_NOOP( "SPLIT" );
const char * TransactionEditor::DEATEXT=QT_TR_NOOP( "<double entry account>" );
const char * TransactionEditor::DEP=QT_TR_NOOP( "<deposit>" );
const char * TransactionEditor::WD=QT_TR_NOOP( "<withdrawal>" );


TransactionEditor::TransactionEditor( QHacc * eng, const Account& acct,
																			const Ledger& l, bool de, QWidget * p,
																			const char * n ) : QFrame( p, n ){

	std::ostream * str=0; 
	if( Utils::debug( 6, str ) ) *str<<"creating new transaction editor"<<endl;

  setFrameStyle( QFrame::Box | QFrame::Plain );
  setLineWidth( 1 );
  setMidLineWidth( 1 );
  setBackgroundColor( QColor( "black" ) );

	engine=eng;
	tsplits=0;
	engine->makeTT( atrans );

	ledger=l;
  account=acct;
  doubleEntry=de;
	connected=false;
	num=credit=debit=0;
	payee=deAccount=0;
	memo=0;
	date=0;
	reco=0;
	split=okay=0;


  // transaction types
  num=new QHaccChoiceEdit( true, this );
	num->setEmptyText( tr( "<num>" ) );

	QStringList sts=QStringList::split( " ", acct.gets( QHacc::ATRANSNUMS ),
																			false );
	num->insertStringList( sts );

  // date field  
  date=new QHaccDateEdit( this );

  // payee field
  payee=new QHaccChoiceEdit( true, this );
	payee->setEmptyText( tr( "<payee>" ) );
	payee->ignoreCase( engine->getBP( "AUTOCOMPNOCASE" ) );
	makePayees();

  // amount field--use qhaccchoiceedit to capture up/down arrows
  credit=new QHaccChoiceEdit( true, this );
  credit->setAlignment( AlignRight );
	credit->setEmptyText( tr( DEP ) );

  debit=new QHaccChoiceEdit( true, this );
  debit->setAlignment( AlignRight );
	debit->setEmptyText( tr( WD ) );

  // reconcile checkbox
  reco=new QCheckBox( this );

  // memo field
  memo=new QHaccLineEdit( this );
	memo->setEmptyText( tr( "<memo>" ) );

  // double entry transaction
  if ( doubleEntry ){
    deAccount=new QHaccChoiceEdit( false, this );
		deAccount->setEmptyText( tr( DEATEXT ) );
		deAccount->ignoreCase( engine->getBP( "AUTOCOMPNOCASE" ) );
		deAccount->setFrame( false );
		connect( deAccount, SIGNAL( returnPressed() ),
						 SLOT( conditionallyAccept() ) );
		connect( engine, SIGNAL( addedA( const Account& ) ),
						 SLOT( makeDAccts() ) );
		connect( engine, SIGNAL( removedA( const Account& ) ),
						 SLOT( makeDAccts() ) );
		connect( engine, SIGNAL( updatedA( const Account&, const Account& ) ),
						 SLOT( makeDAccts() ) );
		makeDAccts();
		
    okay=new QPushButton( tr( "Enter", "transeditor" ), this );
    connect( okay, SIGNAL( clicked() ), SLOT( conditionallyAccept() ) ); 
		
    split=new QPushButton( tr( "Split", "transeditor" ), this );
    connect( split, SIGNAL( clicked() ), SLOT( openSplitEditor() ) );
  }
	else setTabOrder( payee, memo );
  
  QLineEdit * edits[]={ num, date, payee, memo, credit, debit };
  for ( int i=0; i<6; i++ ){
		connect( edits[i], SIGNAL( returnPressed() ),
						 SLOT( conditionallyAccept() ) );
		edits[i]->setFrame( false );
  }
}

TransactionEditor::~TransactionEditor(){
	delete atrans;
	std::ostream * str=0; 
	if( Utils::debug( 6, str ) ) *str<<"destroying old transaction editor"<<endl;
}

void TransactionEditor::reLayout( QHeader * header, int w, int h ){
	// resizing a little bigger gives a nice shadow effect
	resize( w, doubleEntry ? h+5 : h+3 );

	// lay out the fields so they take up all the space available
	if( doubleEntry ){
		h=h/2;
		int y=1;
		QWidget * wdgs[]={ num, date, payee, credit, debit, reco };
		int idx=0;
		for( int i=0; i<7; i++ ){
			int x=header->sectionPos( i )-1;
			int w=header->sectionSize( i )-1;
			if( idx==2 ) w+=header->sectionSize( ++i );
			wdgs[idx]->setGeometry( x, y, w, h );
			idx++;
		}
		
		y+=h+1;
		idx=0;
		QWidget * twdgs[]={ split, deAccount, memo, okay };
		for( int i=0; i<6; i++ ){
			int x=header->sectionPos( i )-1;
			int w=header->sectionSize( i )-1;
			if( idx==0 || idx==3 ) w+=header->sectionSize( ++i );
			if( idx==3 ) w+=header->sectionSize( ++i );
			twdgs[idx]->setGeometry( x, y, w, h );
			idx++;
		}
	}
	else{
		QWidget * wdgs[]={ num, date, payee, memo, credit, debit, reco };
		int y=1;
		for( int i=0; i<7; i++ ){
			int x=header->sectionPos( i )-1;
			int w=header->sectionSize( i )-1;
			wdgs[i]->setGeometry( x, y, w, h );
		}
	}
}

void TransactionEditor::makePayees(){
	TableSelect uq( QHacc::TPAYEE, TableCol( "" ), TableSelect::UQ );

	QHaccTable * temp=0;
	if( engine->getBP( "GLOBALTYPEAHEAD" ) ){
		// get all unique payees
		TableSelect crit[]={ uq };
		engine->getTWhere( crit, 1, temp );
	}
	else{
		// get unique payees from only this account
		TableSelect crit[]={ TableSelect( QHacc::TACCTID,
																			account.get( QHacc::AID ) ),	uq };
		engine->getTWhere( crit, 2, temp );
	}

	for( uint i=0; i<temp->rows(); i++ )
		payee->insertItem( temp->at( i ).gets( QHacc::TPAYEE ) );

	//for( uint i=0; i<temp->rows(); i++ ) cout<<"payee insert: "<<temp->at( i ).gets( QHacc::TPAYEE )<<endl;

	delete temp;
}

void TransactionEditor::makeDAccts(){
	QHaccTable * accts=0;
	engine->getAs( false, accts );

	QStringList str;
	for( uint i=0; i<accts->rows(); i++ ){
		TableRow acct=accts->at( i );
		deAccount->insertItem( engine->getFNameOfA( acct ) );
	}
	delete accts;
}

void TransactionEditor::prepare( const Transaction& t, bool oldT, uint rec,
																 bool recPushedIfNew ){
	edit=oldT;
	recstate=rec; // recwins need to halfway reconcile transactions

	//cout<<"preparing for "<<t.toString()<<endl;
	mytrans=t;

	num->setText( t.gets( QHacc::TNUM ) );
	date->setDate( t.getd( QHacc::TDATE ) );
	payee->setText( t.gets( QHacc::TPAYEE ) );
	if( edit ) reco->setChecked( t.getu( QHacc::TRECO )!=QHacc::NREC );
	else reco->setChecked( recPushedIfNew );
	
	setModel( mytrans );

	int complete=engine->getIP( "AUTOCOMPLETION" );
	if( connected ){
		if( complete==QHacc::NEVER || ( complete==QHacc::NEW && edit ) ){
			disconnect( payee, SIGNAL( textChanged( const QString& ) ),
									this, SLOT( fillInForPayee( const QString& ) ) );
			disconnect( credit, SIGNAL( arrowPressed( bool ) ),
									this, SLOT( moveInSum( bool ) ) );
			disconnect( debit, SIGNAL( arrowPressed( bool ) ),
									this, SLOT( moveInSum( bool ) ) );
			connected=false;
		}
	}
	else{
		if( complete==QHacc::ALL || ( complete==QHacc::NEW && !edit ) ){
			connect( payee, SIGNAL( textChanged( const QString& ) ),
							 this, SLOT( fillInForPayee( const QString& ) ) );
			connect( credit, SIGNAL( arrowPressed( bool ) ),
							 this, SLOT( moveInSum( bool ) ) );
			connect( debit, SIGNAL( arrowPressed( bool ) ),
							 this, SLOT( moveInSum( bool ) ) );
			connected=true;
		}
	}

  num->setFocus();	
}

void TransactionEditor::setModel( const Transaction& t ){
	// set the sum, memo, and de fields appropriately for the given transaction

	//cout<<"setting model to "<<t.toString()<<endl;
	memo->setText( t.gets( QHacc::TMEMO ) );
	
	QString summer=t.gets( QHacc::TSUM );
	if( summer.startsWith( "-" ) ){
		credit->setText( "" );
		debit->setText( summer.mid( 1 ) );
	}
	else{
		credit->setText( summer );		
		debit->setText( "" );
	}

	if( tsplits ) delete tsplits;
	engine->getTSplits( t.getu( QHacc::TID ), tsplits );

	if( doubleEntry ){
		QString pfname;
		QHaccView::getPNamesFor( engine, t.getu( QHacc::TID ), pfname, pfname );
		deAccount->setText( pfname );
	}
	else tsplits->clear();
}

void TransactionEditor::setModel(){
	// if modelstart<0, then we don't have any maatches
	// to fill in, so revert to the original trans

	//cout<<"modelpos is "<<modelpos<<" ("<<modelstart<<","<<modelend<<")"<<endl;
	if( modelstart<0 ) setModel( mytrans );
	else{
		// cycle if out of range
		if( modelpos<modelstart ) modelpos=modelend;
		else if( modelpos>modelend ) modelpos=modelstart;

		Transaction model=apidx->at( modelpos );
		if( !model.isNull() ) setModel( model );
	}
}

void TransactionEditor::fillInForPayee( const QString& smodel ){
	// we are given a payee string...find all the transactions that
	// have this payee, and use them for the auto-completion.
	// if we're looking globally, there are a few extra hoops

	delete atrans;
	//cout<<"fillin is "<<smodel<<endl;

	TableSelect atp( QHacc::TPAYEE, TableCol( smodel ) );
	TableSelect sga( QHacc::TACCTID, account.get( QHacc::AID ) );

	if( engine->getBP( "GLOBALTYPEAHEAD" ) ){
		// if we're looking globally, make sure we remove splits that point
		// to this account (they'll be cicular, and will fail the engine's
		// transaction sanity checks

		// get every transaction with this payee
		TableSelect crit[]={ atp };
		engine->getTWhere( crit, 1, atrans );

		// get transactions just from this account
		QHaccTable * acctt=0;
		atrans->getWhere( sga, acctt );

		// delete splitgroups where one split is part of this account
		for( uint i=0; i<acctt->rows(); i++ ){
			atrans->deleteWhere( TableSelect( QHacc::TSPLITGROUP,	acctt->at( i ).get( QHacc::TSPLITGROUP ) ) );
		}
		atrans->load( *acctt );
		delete acctt;
	}
	else{
		// get unique payees from only this account
		TableSelect crit[]={ atp, sga };
		engine->getTWhere( crit, 2, atrans );
	}

	atrans->addIndexOn( QHacc::TDATE, apidx );
	
	/*
	cout<<"atrans is "<<endl;
	for( uint i=0; i<atrans->rows(); i++ )
		cout<<"  "<<apidx->at( i ).toString()<<endl;
	cout<<"end"<<endl;
	*/
	
	modelstart=0;
	modelend=apidx->rows()-1;
	if( modelend<modelstart ) modelstart=-1;
	

	modelpos=modelend; // fillin with the most recent transaction
	setModel();
}

void TransactionEditor::moveInSum( bool up ){
	// cycle up or down, and change fields as appropriate
	if( up ) modelpos--;
	else modelpos++;
	setModel();
}

bool TransactionEditor::verifyOrMakeA( QHacc * engine,
																			 QHaccLineEdit * texter ){
	// set the texter's text to the name of a newly-made
	// account, or don't do anything if the user doesn't
	// want a new account made

	//cout<<"voma: ->"<<texter->text()<<"<-"<<endl;
  if( texter->text().isEmpty() ){
		Account acct;
		AccountDlg qd( engine, acct, 0, 0, true );
		if ( qd.exec() ){
			acct=qd.getAccount();
			texter->setText( engine->getFNameOfA( acct ) );
			if( texter->text().isEmpty() ) return false;
		}
		else return false;
  }

	return true;
}

void TransactionEditor::conditionallyAccept(){
	// make sure we have all the data we need,
	// and if we do, call accept
	bool good=true;

	if( doubleEntry ){
		// check that the deAccount field is filled in
		QString det=deAccount->text();
		if( det.isNull() ) good=verifyOrMakeA( engine, deAccount );

		// if the det is SPLIT, we know tsplits has already been made
		if( good && det!=tr( SPLIT ) ){
			Account acct=engine->getA( deAccount->text() );
			int cr=engine->convMoney( credit->text() );
			int db=engine->convMoney( debit->text() );
						
			Transaction t=engine->getBlankTForAL( acct, ledger );
			t.set( QHacc::TSUM, TableCol( engine->convMoney( db-cr ) ) );
				
			// drop the old split in case the user changed the de acct
			tsplits->clear();
			tsplits->add( t );			
		}
	}
	if( good ) accept();
}

void TransactionEditor::accept(){
	uint tid=mytrans.getu( QHacc::TID );
	uint aid=mytrans.getu( QHacc::TACCTID );
	uint lid=mytrans.getu( QHacc::TLID );

	if( edit ) mytrans=engine->getT( tid );
	else mytrans=engine->getBlankTForAL( engine->getA( aid ), 
																			 engine->getL( lid ) );
	
  // make sure if the user is in the date field, we commit any changes
  date->setDate( date->getDateFromString( date->text() ) );
	
	// figure out the appropriate sum
	int cred=engine->convMoney( credit->text() );
	int debi=engine->convMoney( debit->text() );
	mytrans.set( QHacc::TSUM, TableCol( engine->convMoney( cred-debi ) ) );
	
	if( reco->isChecked() )	mytrans.set( QHacc::TRECO, TableCol( recstate ) );


	TableCol dater( date->getDate() );
	TableCol numer( num->text() );
	TableCol payer( payee->text() );
	TableCol memer( memo->text() );

	// add a COPY of mytrans so closeEvent can safely delete mytrans AND tsplits
	tsplits->add( mytrans );

	tsplits->updateWhere( TableSelect(), QHacc::TNUM,   numer );
	tsplits->updateWhere( TableSelect(), QHacc::TDATE,  dater );
	tsplits->updateWhere( TableSelect(), QHacc::TPAYEE, payer );
	tsplits->updateWhere( TableSelect(), QHacc::TMEMO,  memer );

	if( edit ){
		// if we're editing the trans, the engine will figure
		// out which splits to add/remove/update for us
		engine->updateTSG( mytrans.get( QHacc::TSPLITGROUP ), tsplits );
	}
	else{
		uint sg=engine->addTSG( tsplits );
		QHaccTable * temp=0;
		TableSelect criteria[]={
			TableSelect( QHacc::TSPLITGROUP, TableCol( sg ) ),
			TableSelect( QHacc::TACCTID, mytrans.getu( QHacc::TACCTID ) )
		};
		engine->getTWhere( criteria, 2, temp );

		if( !temp->isEmpty() ) atrans->add( temp->at( 0 ) );
		delete temp;
	}

	payee->insertItem( payee->text() ); // include in type-aheads

	close();
}

void TransactionEditor::closeEvent( QCloseEvent * c ){
  emit closed();

	mytrans=Transaction();
	if( tsplits ) delete tsplits;
	tsplits=0;

  QFrame::closeEvent( c );
}

void TransactionEditor::openSplitEditor(){
	SplitEditor se( engine, tsplits, this );
	if( se.exec() ){
		// we get a table that just has account ids and sums,
		// we need to convert this to actual transactions, and make
		// sure that the splits are in a consistent state of summation
		QHaccTable * temp=0;
		se.getSplits( temp );

		// cycle through the splits, and see which we
		// should keep, which need to get made.
		QHaccTable splits( QHacc::TCOLTYPES, QHacc::TCOLNAMES, QHacc::TCOLS,
											 -1, 0, temp->rows(), 1, 1 );

		for( uint i=0; i<temp->rows(); i++ ){
			Transaction t=temp->at( i ), st;
			uint ti=0;
			
			// if the split already exists, just edit it
			// else make a new split and edit that
			if( tsplits->contains( t, ti ) ) st=tsplits->at( ti );
			else st=engine->getBlankTForAL( account, ledger );
			
			st.set( QHacc::TSUM, t.get( SSUM ) );
			st.set( QHacc::TACCTID, t.get( SACCT ) );
			
			splits.add( st );
		}
		
		delete tsplits;
		tsplits=new QHaccTable( splits );		

		// this is where we make sure DEBITS=CREDITS
		uint rows=tsplits->rows();
		//float credits=0, debits=0 ;
		int credits=0, debits=0 ;
		for( uint i=0; i<rows; i++ ){
			TableRow split=tsplits->at( i );
			int tsum=engine->convMoney( split.gets( QHacc::TSUM ) );
			if( tsum>0 ) debits+=tsum;
			else credits-=tsum; // this is actually a sign-flip
		}
		
		//float summer=credits-debits;
		int summer=credits-debits;
		if( summer>=0 ){
			credit->setText( engine->convMoney( summer ) );
			//credit->setText( QString().setNum( summer, 'f', QHacc::DECIMALS ) );
			debit->setText( "" );
		}
		else{
			summer=0-summer;
			credit->setText( "" );
			//debit->setText( QString().setNum( summer, 'f', QHacc::DECIMALS ) );
			debit->setText( engine->convMoney( summer ) );
		}
		
		QString det;
		if( rows==1 ){
			TableRow row=tsplits->at( 0 );
			det=engine->getFNameOfA( engine->getA( row.getu( QHacc::TACCTID ) ) );
		}
		else if( rows>1 ) det=tr( SPLIT );
		deAccount->setText( det );
	}
	else setModel( mytrans );
}

void TransactionEditor::keyPressEvent( QKeyEvent * qke ){
	if( qke->key()==Key_Escape ) close();
}

/* * * * * * * * * * * * * * * * * * *
 * * * * * SPLIT TRANSACTION * * * * *
 * * * * * * * * * * * * * * * * * * */
SplitEditor::SplitEditor( QHacc * engine, QHaccTable * splits, QWidget * p,
													const char * n ) : QDialog( p, n, true ){

  resize( 420, 200 );
	setMinimumSize( 420, 200 );
	setMaximumSize( 420, 200 );
  setCaption( tr( "QHacc Split Editor" ) );

  area=new AccountArea( engine, splits, this );
  QGridLayout * layout=new QGridLayout( this, 5, 5 );	
  layout->addMultiCellWidget( area, 0, 3, 0, 4 );
	
  QPushButton * but=new QPushButton( tr( "okay", "split pushbutton" ), this );
  connect( but, SIGNAL( clicked() ), SLOT( accept() ) );
  layout->addMultiCellWidget( but, 4, 4, 0, 1 ); 
	
  but=new QPushButton( tr( "cancel", "split pushbutton" ), this );
  connect( but, SIGNAL( clicked() ), SLOT( reject() ) );
  layout->addMultiCellWidget( but, 4, 4, 2, 3 ); 

  but=new QPushButton( tr( "add split", "split pushbutton" ), this );
  connect( but, SIGNAL( clicked() ), SLOT( addSplitToArea() ) );
  layout->addMultiCellWidget( but, 4, 4, 4, 4 );

}

SplitEditor::~SplitEditor(){}

void SplitEditor::getSplits( QHaccTable *& splits ){
	area->getSplits( splits );
}

void SplitEditor::addSplitToArea(){ area->addSplit( Transaction() ); }

/* * * * * * * * * * * * * * * * * * *
 * * * * * * * ACCOUNTAREA * * * * * *
 * * * * * * * * * * * * * * * * * * */

AccountArea::AccountArea( QHacc * eng, QHaccTable * splits, QWidget * p,
													const char * n ) : QScrollView( p, n ){
	engine=eng;
  setMargins( 0, 20, 0, 0 );
  setVScrollBarMode( AlwaysOn );

  header=new QHeader( this );
  header->setMovingEnabled( false );
  header->resize( 400, 20 );
  header->addLabel( tr( "Account", "editor heading" ), 200 );
  header->addLabel( tr( "Deposit", "editor heading" ), 100 );
  header->addLabel( tr( "Withdrawal", "editor heading" ), 100 );
  header->setResizeEnabled( false );
  header->setClickEnabled( false );

	accounts.setAutoDelete( true );
	credits.setAutoDelete( true );
	debits.setAutoDelete( true );


	uint rows=splits->rows();
	for( uint i=0; i<rows; i++ ){
		TableRow row=splits->at( i );
		TableCol cols[]={ TableCol( row.getu( QHacc::TACCTID ) ),
											TableCol( row.gets( QHacc::TSUM ) ) };
		addSplit( TableRow( cols, TransactionEditor::SCOLS ) );
	}

	// have a minimum of 5 splits available initially, and at least 1 empty one
	for( uint i=rows; i<4; i++ ) addSplit( Transaction() );
	addSplit( Transaction() );
	accounts.at( rows )->setFocus();
}

AccountArea::~AccountArea(){}

void AccountArea::addSplit( const TableRow& trans ){
  uint y=getNumSplits()*lineSize();

	//if( row ) cout<<" addSplit: "<<row->toString()<<endl;
  QHaccChoiceEdit * edit=new QHaccChoiceEdit( false, this );
	edit->ignoreCase( engine->getBP( "AUTOCOMPNOCASE" ) );
	edit->setEmptyText( tr( TransactionEditor::DEATEXT ) );

	QHaccTable * accts=0;
	engine->getAs( false, accts );
	QStringList str;
	QString atext;
	uint aid=( trans.isNull() ? 0 : trans.getu( TransactionEditor::SACCT  ) );

	// build the list of account choices
	for( uint i=0; i<accts->rows(); i++ ){
		TableRow acct=accts->at( i );
		QString stringy=engine->getFNameOfA( acct );
		edit->insertItem( stringy );
		// if we find our account id, remember the acctname
		if( acct.compareTo( aid, QHacc::AID )==0 )	atext=stringy;
	}
	delete accts;
	
  edit->insertStringList( str );
	edit->setText( atext );

	int ls=lineSize();
	
  edit->resize( header->sectionSize( 0 ), ls );

  QHaccLineEdit * credit=new QHaccLineEdit( this );
  credit->resize( header->sectionSize( 1 ), ls );
	credit->setEmptyText( tr( TransactionEditor::DEP ) );

  QHaccLineEdit * debit=new QHaccLineEdit( this );
  debit->resize( header->sectionSize( 2 ), ls );
	debit->setEmptyText( tr( TransactionEditor::WD ) );

	QString stext;
	if( !trans.isNull() ) {
		int summer=engine->convMoney( trans.gets( TransactionEditor::SSUM ) );
		if( summer<0 ){
			stext=engine->convMoney( 0-summer );
			debit->setText( stext );
		}
		else{
			stext=engine->convMoney( summer );
			credit->setText( stext );
		}
	}

  accounts.append( edit );
  credits.append( credit );  
  debits.append( debit );  

	resizeContents( 400, getNumSplits()*ls );
	QLineEdit * wdgs[]={ edit, credit, debit };
	for( uint i=0; i<3; i++ ){
		addChild( wdgs[i], header->sectionPos( i ), y );
		wdgs[i]->show();
	}
  edit->setFocus();
}

uint AccountArea::getNumSplits() const { return accounts.count(); }
int AccountArea::lineSize() const { return fontMetrics().height()+5; }

void AccountArea::getSplits( QHaccTable *& splits ){
	uint nspl=getNumSplits();

	splits=new QHaccTable( TransactionEditor::SCOLTYPES,
												 TransactionEditor::SCOLNAMES,
												 TransactionEditor::SCOLS,
												 -1, 0, nspl, 0, 0 );

  for ( uint i=0; i<nspl; i++ ){
		bool cont=true;
		
		int cr=engine->convMoney( credits.at( i )->text() );
		int db=engine->convMoney( debits.at( i )->text() );

		if( cr-db!=0 ){
			QHaccLineEdit * acctname=accounts.at( i );
			QString aval=acctname->text();

			// make any necessary accounts
			if( aval.isEmpty() ){
				cont=TransactionEditor::verifyOrMakeA( engine, acctname );
				if( cont ) aval=acctname->text();
				else cont=false;
			}

			if( cont ){
				Account acct=engine->getA( aval );
				TableCol cols[]={ TableCol( acct.getu( QHacc::AID ) ),
													TableCol( engine->convMoney( cr-db ) ) };
				splits->add( TableRow( cols, TransactionEditor::SCOLS ) );
			}
		}
	}
}

